@@ -48,7 +48,9 @@ static unsigned int off;
 static void elf_freebsd_probe(void)
 {
 	image_type = Elf;
-	if (estate.e.elf32.e_entry & 0xf0000000) {
+	if (	(estate.e.elf32.e_entry & 0xf0000000) && 
+		(estate.e.elf32.e_type == ET_EXEC))
+	{
 		image_type = Elf_FreeBSD;
 		printf("/FreeBSD");
 		off = -(estate.e.elf32.e_entry & 0xff000000);
@@ -214,7 +216,7 @@ static int elf_freebsd_debug_loader(unsigned int offset)
 					estate.toread, estate.curaddr);
 #endif
 				/* Save where we are loading this... */
-				symtab_load = estate.curaddr;
+				symtab_load = phys_to_virt(estate.curaddr);
 				
 				*((long *)phys_to_virt(estate.curaddr)) = estate.toread;
 				estate.curaddr += sizeof(long);
@@ -242,7 +244,7 @@ static int elf_freebsd_debug_loader(unsigned int offset)
 					estate.toread, estate.curaddr);
 #endif
 				/* Save where we are loading this... */
-				symstr_load = estate.curaddr;
+				symstr_load = phys_to_virt(estate.curaddr);
 				
 				*((long *)phys_to_virt(estate.curaddr)) = estate.toread;
 				estate.curaddr += sizeof(long);
@@ -288,7 +290,7 @@ static void elf_freebsd_boot(unsigned long entry)
 		/* Assumes size of long is a power of 2... */
 		bsdinfo.bi_esymtab = (symstr_load +
 			sizeof(long) +
-			*((long *)phys_to_virt(symstr_load)) +
+			*((long *)symstr_load) +
 			sizeof(long) - 1) & ~(sizeof(long) - 1);
 		
 		/* Where we will build the meta data... */
