@@ -6,7 +6,7 @@ This file Copyright (C) 2003 Michael Brown <mbrown@fensystems.co.uk>
 of Fen Systems Ltd. (http://www.fensystems.co.uk/).  All rights
 reserved.
 
-$Id: undi.c,v 1.8 2003/10/25 13:54:53 mcb30 Exp $
+$Id: undi.c,v 1.18 2005/02/16 08:37:01 ebiederm Exp $
 ***************************************************************************/
 
 /*
@@ -28,21 +28,65 @@ $Id: undi.c,v 1.8 2003/10/25 13:54:53 mcb30 Exp $
 #include "undi.h"
 /* 8259 PIC defines */
 #include "pic8259.h"
+/* Real-mode calls */
+#include "realmode.h"
+/* E820 map mangler */
+#include "hidemem.h"
 
 /* NIC specific static variables go here */
-static undi_t undi = { NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-		       NULL, NULL, 0, NULL, 0, NULL,
-		       0, 0, 0, 0,
-		       { 0, 0, 0, NULL, 0, 0, 0, 0, 0, NULL },
-		       IRQ_NONE };
+static undi_t undi = { 
+	.pnp_bios         = NULL, 
+	.rom              = NULL, 
+	.undi_rom_id      = NULL, 
+	.pxe              = NULL, 
+	.pxs              = NULL, 
+	.xmit_data        = NULL,
+	.base_mem_data    = NULL, 
+	.driver_code      = NULL, 
+	.driver_code_size = 0, 
+	.driver_data      = NULL, 
+	.driver_data_size = 0, 
+	.xmit_buffer      = NULL,
+	.prestarted       = 0, 
+	.started          = 0, 
+	.initialized      = 0, 
+	.opened           = 0,
+	.irq              = IRQ_NONE 
+};
 
 /* Function prototypes */
-int allocate_base_mem_data ( void );
-int free_base_mem_data ( void );
-int eb_pxenv_undi_shutdown ( void );
-int eb_pxenv_stop_undi ( void );
-int undi_unload_base_code ( void );
-int undi_full_shutdown ( void );
+static int allocate_base_mem_data ( void );
+static int free_base_mem_data ( void );
+static int eb_pxenv_undi_shutdown ( void );
+static int eb_pxenv_stop_undi ( void );
+static int undi_unload_base_code ( void );
+static int undi_full_shutdown ( void );
+
+/* Trivial/nontrivial IRQ handler selection */
+#if defined(UNDI_NONTRIVIAL_IRQ)
+static void nontrivial_irq_handler ( void );
+static void nontrivial_irq_handler_end ( void );
+static int install_nontrivial_irq_handler ( irq_t irq );
+static int remove_nontrivial_irq_handler ( irq_t irq );
+static int nontrivial_irq_triggered ( irq_t irq );
+static int copy_nontrivial_irq_handler ( void *target, size_t target_size );
+#define NONTRIVIAL_IRQ_HANDLER_SIZE FRAGMENT_SIZE(nontrivial_irq_handler)
+#define install_undi_irq_handler(irq) install_nontrivial_irq_handler(irq)
+#define remove_undi_irq_handler(irq) remove_nontrivial_irq_handler(irq)
+#define undi_irq_triggered(irq) nontrivial_irq_triggered(irq)
+#define UNDI_IRQ_HANDLER_SIZE NONTRIVIAL_IRQ_HANDLER_SIZE
+#define copy_undi_irq_handler(dest,size) copy_nontrivial_irq_handler(dest,size)
+#else
+#define install_undi_irq_handler(irq) install_trivial_irq_handler(irq)
+#define remove_undi_irq_handler(irq) remove_trivial_irq_handler(irq)
+#define undi_irq_triggered(irq) trivial_irq_triggered(irq)
+#define UNDI_IRQ_HANDLER_SIZE TRIVIAL_IRQ_HANDLER_SIZE
+#define copy_undi_irq_handler(dest,size) copy_trivial_irq_handler(dest,size)
+#endif /* UNDI_NONTRIVIAL_IRQ */
+
+/* Size of variable-length data in base_mem_data */
+#define BASE_MEM_VARDATA_SIZE ( UNDI_IRQ_HANDLER_SIZE > e820mangler_size ? \
+				UNDI_IRQ_HANDLER_SIZE : e820mangler_size )
 
 /**************************************************************************
  * Utility functions
@@ -51,7 +95,7 @@ int undi_full_shutdown ( void );
 /* Checksum a block.
  */
 
-uint8_t checksum ( void *block, size_t size ) {
+static uint8_t checksum ( void *block, size_t size ) {
 	uint8_t sum = 0;
 	uint16_t i = 0;
 	for ( i = 0; i < size; i++ ) {
@@ -63,7 +107,8 @@ uint8_t checksum ( void *block, size_t size ) {
 /* Print the status of a !PXE structure
  */
 
-void pxe_dump ( void ) {
+static void pxe_dump ( void )
+{
 	printf ( "API %hx:%hx St %hx:%hx UD %hx:%hx UC %hx:%hx "
 		 "BD %hx:%hx BC %hx:%hx\n",
 		 undi.pxe->EntryPointSP.segment, undi.pxe->EntryPointSP.offset,
@@ -77,46 +122,42 @@ void pxe_dump ( void ) {
 /* Allocate/free space for structures that must reside in base memory
  */
 
-int allocate_base_mem_data ( void ) {
+static int allocate_base_mem_data ( void ) {
 	/* Allocate space in base memory.
 	 * Initialise pointers to base memory structures.
 	 */
 	if ( undi.base_mem_data == NULL ) {
 		undi.base_mem_data =
 			allot_base_memory ( sizeof(undi_base_mem_data_t) +
-					    TRIVIAL_IRQ_HANDLER_SIZE );
+					    BASE_MEM_VARDATA_SIZE );
 		if ( undi.base_mem_data == NULL ) {
 			printf ( "Failed to allocate base memory\n" );
 			free_base_mem_data();
 			return 0;
 		}
 		memset ( undi.base_mem_data, 0, sizeof(undi_base_mem_data_t) );
-		undi.undi_call_info = &undi.base_mem_data->undi_call_info;
 		undi.pxs = &undi.base_mem_data->pxs;
 		undi.xmit_data = &undi.base_mem_data->xmit_data;
 		undi.xmit_buffer = undi.base_mem_data->xmit_buffer;
-		copy_trivial_irq_handler ( undi.base_mem_data->irq_handler,
-					   TRIVIAL_IRQ_HANDLER_SIZE );
 	}
 	return 1;
 }
 
-int free_base_mem_data ( void ) {
+static int free_base_mem_data ( void ) {
 	if ( undi.base_mem_data != NULL ) {
 		forget_base_memory ( undi.base_mem_data,
 				     sizeof(undi_base_mem_data_t) +
-				     TRIVIAL_IRQ_HANDLER_SIZE );
+				     BASE_MEM_VARDATA_SIZE );
 		undi.base_mem_data = NULL;
-		undi.undi_call_info = NULL;
 		undi.pxs = NULL;
 		undi.xmit_data = NULL;
 		undi.xmit_buffer = NULL;
-		copy_trivial_irq_handler ( NULL, 0 );
+		copy_undi_irq_handler ( NULL, 0 );
 	}
 	return 1;
 }
 
-void assemble_firing_squad ( firing_squad_lineup_t *lineup,
+static void assemble_firing_squad ( firing_squad_lineup_t *lineup,
 			     void *start, size_t size,
 			     firing_squad_shoot_t shoot ) {
 	int target;
@@ -133,7 +174,7 @@ void assemble_firing_squad ( firing_squad_lineup_t *lineup,
 	}
 }
 
-void shoot_targets ( firing_squad_lineup_t *lineup ) {
+static void shoot_targets ( firing_squad_lineup_t *lineup ) {
 	int shoot_this_target = 0;
 	int shoot_last_target = 0;
 	int start_target = 0;
@@ -175,7 +216,7 @@ void shoot_targets ( firing_squad_lineup_t *lineup ) {
 /* Locate the $PnP structure indicating a PnP BIOS.
  */
 
-int hunt_pnp_bios ( void ) {
+static int hunt_pnp_bios ( void ) {
 	uint32_t off = 0x10000;
 
 	printf ( "Hunting for PnP BIOS..." );
@@ -200,7 +241,7 @@ int hunt_pnp_bios ( void ) {
 /* Locate the !PXE structure indicating a loaded UNDI driver.
  */
 
-int hunt_pixie ( void ) {
+static int hunt_pixie ( void ) {
 	static uint32_t ptr = 0;
 	pxe_t *pxe = NULL;
 
@@ -245,7 +286,8 @@ int hunt_pixie ( void ) {
 /* Locate PCI PnP ROMs.
  */
 
-int hunt_rom ( void ) {
+static int hunt_rom ( struct pci_device *pci ) 
+{
 	static uint32_t ptr = 0;
 
 	printf ( "Hunting for ROMs..." );
@@ -270,11 +312,11 @@ int hunt_rom ( void ) {
 			}
 			printf ( "PCI:%hx:%hx...", pcir_header->vendor_id,
 				 pcir_header->device_id );
-			if ( ( pcir_header->vendor_id != undi.pci.vendor ) ||
-			     ( pcir_header->device_id != undi.pci.dev_id ) ) {
+			if ( ( pcir_header->vendor_id != pci->vendor ) ||
+			     ( pcir_header->device_id != pci->dev_id ) ) {
 				printf ( "not me (%hx:%hx)\n...",
-					 undi.pci.vendor,
-					 undi.pci.dev_id );
+					pci->vendor,
+					pci->dev_id );
 				continue;
 			}
 			if ( undi.rom->pnp_off == 0 ) {
@@ -308,8 +350,9 @@ int hunt_rom ( void ) {
 /* Locate ROMs containing UNDI drivers.
  */
 
-int hunt_undi_rom ( void ) {
-	while ( hunt_rom() ) {
+static int hunt_undi_rom ( struct pci_device *pci ) 
+{
+	while ( hunt_rom(pci) ) {
 		if ( undi.rom->undi_rom_id_off == 0 ) {
 			printf ( "Not a PXE ROM\n" );
 			continue;
@@ -320,6 +363,11 @@ int hunt_undi_rom ( void ) {
 			printf ( "Invalid UNDI signature\n" );
 			continue;
 		}
+		if ( checksum ( undi.undi_rom_id,
+				undi.undi_rom_id->struct_length ) != 0 ) {
+			printf ( "Invalid checksum\n" );
+			continue;
+		}
 		printf ( "Located UNDI ROM supporting revision %d.%d.%d\n",
 			 undi.undi_rom_id->undi_rev[2],
 			 undi.undi_rom_id->undi_rev[1],
@@ -336,21 +384,36 @@ int hunt_undi_rom ( void ) {
 /* Make a real-mode UNDI API call to the UNDI routine at
  * routine_seg:routine_off, passing in three uint16 parameters on the
  * real-mode stack.
- * Calls the assembler wrapper routine __undi_call.
  */
 
-static inline PXENV_EXIT_t _undi_call ( uint16_t routine_seg,
-					uint16_t routine_off, uint16_t st0,
-					uint16_t st1, uint16_t st2 ) {
+static PXENV_EXIT_t _undi_call ( uint16_t routine_seg,
+			  uint16_t routine_off, uint16_t st0,
+			  uint16_t st1, uint16_t st2 ) {
 	PXENV_EXIT_t ret = PXENV_EXIT_FAILURE;
+	struct {
+		segoff_t routine;
+		uint16_t st0;
+		uint16_t st1;
+		uint16_t st2;
+	} PACKED in_stack = {
+		{ routine_off, routine_seg }, st0, st1, st2
+	};
 
-	undi.undi_call_info->routine.segment = routine_seg;
-	undi.undi_call_info->routine.offset = routine_off;
-	undi.undi_call_info->stack[0] = st0;
-	undi.undi_call_info->stack[1] = st1;
-	undi.undi_call_info->stack[2] = st2;
-	ret = __undi_call ( SEGMENT( undi.undi_call_info ),
-			    OFFSET( undi.undi_call_info ) );
+	RM_FRAGMENT(rm_undi_call, 
+		"popw %di\n\t"			/* %es:di = routine */
+		"popw %es\n\t"
+		"pushw %cs\n\t"			/* set up return address */
+		"call 1f\n\t1:popw %bx\n\t"
+		"leaw (2f-1b)(%bx), %ax\n\t"
+		"pushw %ax\n\t"
+		"pushw %es\n\t"			/* routine address to stack */
+		"pushw %di\n\t"
+		"lret\n\t"			/* calculated lcall */
+		"\n2:\n\t"			/* continuation point */
+	);
+
+	/* Parameters are left on stack: set out_stack = in_stack */
+	ret = real_call ( rm_undi_call, &in_stack, &in_stack );
 
 	/* UNDI API calls may rudely change the status of A20 and not
 	 * bother to restore it afterwards.  Intel is known to be
@@ -370,14 +433,24 @@ static inline PXENV_EXIT_t _undi_call ( uint16_t routine_seg,
  * pxenv_structure on the real-mode stack.
  */
 
-int undi_call_loader ( void ) {
+static int undi_call_loader ( void ) {
 	PXENV_EXIT_t pxenv_exit = PXENV_EXIT_FAILURE;
 	
+	/* Hide Etherboot around the loader, so that the PXE stack
+	 * doesn't trash our memory areas
+	 */
+	install_e820mangler ( undi.base_mem_data->e820mangler );
+	hide_etherboot();
 	pxenv_exit = _undi_call ( SEGMENT( undi.rom ),
 				  undi.undi_rom_id->undi_loader_off,
 				  OFFSET( undi.pxs ),
 				  SEGMENT( undi.pxs ),
 				  0 /* Unused for UNDI loader API */ );
+	if ( !unhide_etherboot() ) {
+		printf ( "FATAL: corrupt INT15\n" );
+		return 0;
+	}
+
 	/* Return 1 for success, to be consistent with other routines */
 	if ( pxenv_exit == PXENV_EXIT_SUCCESS ) return 1;
 	printf ( "UNDI loader call failed with status %#hx\n",
@@ -392,7 +465,7 @@ int undi_call_loader ( void ) {
  * codes, undi_call_silent() will not.
  */
 
-int undi_call_silent ( uint16_t opcode ) {
+static int undi_call_silent ( uint16_t opcode ) {
 	PXENV_EXIT_t pxenv_exit = PXENV_EXIT_FAILURE;
 
 	pxenv_exit = _undi_call ( undi.pxe->EntryPointSP.segment,
@@ -404,13 +477,190 @@ int undi_call_silent ( uint16_t opcode ) {
 	return pxenv_exit == PXENV_EXIT_SUCCESS ? 1 : 0;
 }
 
-int undi_call ( uint16_t opcode ) {
+static int undi_call ( uint16_t opcode ) {
 	if ( undi_call_silent ( opcode ) ) return 1;
 	printf ( "UNDI API call %#hx failed with status %#hx\n",
 		 opcode, undi.pxs->Status );
 	return 0;
 }
 
+#ifdef UNDI_NONTRIVIAL_IRQ
+/* IRQ handler that actually calls PXENV_UNDI_ISR.  It's probably
+ * better to use the trivial IRQ handler, since this seems to work for
+ * just about all known NICs and doesn't involve making a PXE API call
+ * in interrupt context.
+ *
+ * This routine is mainly used for testing the Etherboot PXE stack's
+ * ability to be called in interrupt context.  It is not compiled in
+ * by default.
+ *
+ * This code has fewer safety checks than those in the
+ * trivial_irq_handler routines.  These are omitted because this code
+ * is not intended for mainstream use.
+ */
+
+
+uint16_t nontrivial_irq_previous_trigger_count = 0;
+
+static int copy_nontrivial_irq_handler ( void *target, size_t target_size __unused )
+{
+	RM_FRAGMENT(nontrivial_irq_handler,
+	/* Will be installed on a paragraph boundary, so access variables
+	 * using %cs:(xxx-irqstart)
+	 */
+		"irqstart:\n\t"
+		"\n\t"
+	/* Fields here must match those in undi_irq_handler_t */
+		"chain_to:\t.word 0,0\n\t"
+		"irq_chain:\t.byte 0,0,0,0\n\t"
+		"entry:\t.word 0,0\n\t"
+		"count_all:\t.word 0\n\t"
+		"count_ours:\t.word 0\n\t"
+		"undi_isr:\n\t"
+		"undi_isr_Status:\t.word 0\n\t"
+		"undi_isr_FuncFlag:\t.word 0\n\t"
+		"undi_isr_others:\t.word 0,0,0,0,0,0\n\t"
+		"handler:\n\t"
+	/* Assume that PXE stack will corrupt everything */
+		"pushal\n\t"
+		"push %ds\n\t"
+		"push %es\n\t"
+		"push %fs\n\t"
+		"push %gs\n\t"
+	/* Set DS == CS */	
+		"pushw %cs\n\t"
+		"popw %ds\n\t"
+	/* Set up parameters for call */
+		"movw $" RM_STR(PXENV_UNDI_ISR_IN_START) ", %ds:(undi_isr_FuncFlag-irqstart)\n\t"
+		"pushw %cs\n\t"
+		"popw %es\n\t"
+		"movw $(undi_isr-irqstart), %di\n\t"
+		"movw $" RM_STR(PXENV_UNDI_ISR) ", %bx\n\t"
+		"pushw %es\n\t"    /* Registers for PXENV+, stack for !PXE */
+		"pushw %di\n\t"
+		"pushw %bx\n\t"
+	/* Make PXE API call */
+		"lcall *%ds:(entry-irqstart)\n\t"
+		"addw $6, %sp\n\t"
+	/* Set DS == CS */	
+		"pushw %cs\n\t"
+		"popw %ds\n\t"
+	/* Check return status to see if it's one of our interrupts */
+		"cmpw $" RM_STR(PXENV_STATUS_SUCCESS) ", %cs:(undi_isr_Status-irqstart)\n\t"
+		"jne 1f\n\t"
+		"cmpw $" RM_STR(PXENV_UNDI_ISR_OUT_OURS) ", %cs:(undi_isr_FuncFlag-irqstart)\n\t"
+		"jne 1f\n\t"
+	/* Increment count_ours if so */
+		"incw %ds:(count_ours-irqstart)\n\t"
+		"1:\n\t"
+	/* Increment count_all anyway */
+		"incw %ds:(count_all-irqstart)\n\t"
+	/* Restore registers and return */
+		"popw %gs\n\t"
+		"popw %fs\n\t"
+		"popw %es\n\t"
+		"popw %ds\n\t"
+		"popal\n\t"
+		"\n\t"
+	/* Chain to acknowledge the interrupt */
+		"cmpb $0, %cs:(irq_chain-irqstart)\n\t"
+		"jz 2f\n\t"
+		"ljmp %cs:(chain_to-irqstart)\n\t"
+		"2:\n\t"
+		"\n\t"
+		"iret\n\t"
+		"\n\t"
+	);
+
+	/* Copy handler */
+	memcpy ( target, nontrivial_irq_handler, NONTRIVIAL_IRQ_HANDLER_SIZE );
+
+	return 1;
+}
+
+static int install_nontrivial_irq_handler ( irq_t irq )
+{
+	undi_irq_handler_t *handler = 
+		&undi.base_mem_data->nontrivial_irq_handler;
+	segoff_t isr_segoff;
+
+	printf ( "WARNING: using non-trivial IRQ handler [EXPERIMENTAL]\n" );
+
+	if (OFFSET(handler) != 0) {
+		DBG("handler not paragraph aligned\n");
+		return 0;
+	}
+	
+	handler->count_all = 0;
+	handler->count_ours = 0;
+	handler->entry = undi.pxe->EntryPointSP;
+	nontrivial_irq_previous_trigger_count = 0;
+	isr_segoff.segment = SEGMENT(handler);
+	isr_segoff.offset = (void*)&handler->code - (void*)handler;
+	install_irq_handler( irq, &isr_segoff, 
+		&handler->irq_chain, &handler->chain_to);
+#if defined(TRACE_UNDI) && (TRACE_UNDI > 0)
+	DBG ( "Testing nontrivial IRQ handler for irq: %d\n", irq );
+
+	if (irq == 0) {
+		/* If it is the timer interrupt we don't have to trigger anything
+		 * we can just wait for it to occur,
+		 */
+		unsigned long start_ticks, ticks;
+		start_ticks = ticks = currticks();
+		do {
+			printf("^");
+			ticks = currticks();
+		} while(ticks == start_ticks);
+	} else {
+		fake_irq ( irq );
+	}
+	if ( handler->count_all == 0 ) {
+		DBG ( "Installation of non-trivial IRQ handler failed\n" );
+		remove_nontrivial_irq_handler ( irq );
+		return 0;
+	}
+#endif
+	DBG ( "Non-Trivial IRQ handler installed successfully\n" );
+
+	return 1;
+}
+
+static int remove_nontrivial_irq_handler ( irq_t irq )
+{
+	undi_irq_handler_t *handler = 
+		&undi.base_mem_data->nontrivial_irq_handler;
+	segoff_t isr_segoff;
+
+	isr_segoff.segment = SEGMENT(handler);
+	isr_segoff.offset = (char*)&handler->code - (char*)handler;
+	remove_irq_handler( irq, &isr_segoff, 
+		&handler->irq_chain, &handler->chain_to);
+	return 1;
+}
+
+static int nontrivial_irq_triggered ( irq_t irq __unused )
+{
+	undi_irq_handler_t *handler = 
+		&undi.base_mem_data->nontrivial_irq_handler;
+	uint16_t count = handler->count_ours;
+	int triggered = count -  nontrivial_irq_previous_trigger_count;
+
+	nontrivial_irq_previous_trigger_count = count;
+
+	return triggered ? 1 : 0;
+}
+
+static void nontrivial_irq_debug ( irq_t irq )
+{
+	undi_irq_handler_t *handler = 
+		&undi.base_mem_data->nontrivial_irq_handler;
+
+	printf ( "IRQ %d triggered %d times (%d of which were ours)\n",
+		 irq, handler->count_all, handler->count_ours );
+}
+#endif /* UNDI_NONTRIVIAL_IRQ */
+	
 /**************************************************************************
  * High-level UNDI API call wrappers
  **************************************************************************/
@@ -418,11 +668,12 @@ int undi_call ( uint16_t opcode ) {
 /* Install the UNDI driver from a located UNDI ROM.
  */
 
-int undi_loader ( void ) {
+static int undi_loader(struct pci_device *pci) 
+{
 	pxe_t *pxe = NULL;
 
 	/* AX contains PCI bus:devfn (PCI specification) */
-	undi.pxs->loader.ax = ( undi.pci.bus << 8 ) | undi.pci.devfn;
+	undi.pxs->loader.ax = ( pci->bus << 8 ) | pci->devfn;
 	/* BX and DX set to 0xffff for non-ISAPnP devices
 	 * (BIOS boot specification)
 	 */
@@ -431,8 +682,14 @@ int undi_loader ( void ) {
 	/* ES:DI points to PnP BIOS' $PnP structure
 	 * (BIOS boot specification)
 	 */
-	undi.pxs->loader.es = 0xf000;
-	undi.pxs->loader.di = virt_to_phys ( undi.pnp_bios ) - 0xf0000;
+	if ( undi.pnp_bios ) {
+		undi.pxs->loader.es = 0xf000;
+		undi.pxs->loader.di = virt_to_phys ( undi.pnp_bios ) - 0xf0000;
+	} else {
+		/* Set to a NULL pointer and hope that we don't need it */
+		undi.pxs->loader.es = 0x0000;
+		undi.pxs->loader.di = 0x0000;
+	}
 
 	/* Allocate space for UNDI driver's code and data segments */
 	undi.driver_code_size = undi.undi_rom_id->code_size;
@@ -459,9 +716,10 @@ int undi_loader ( void ) {
 	/* Do the API call to install the loader */
 	if ( ! undi_call_loader () ) return 0;
 
-	pxe = VIRTUAL( undi.pxs->loader.undi_cs, undi.pxs->loader.pxe_off );
+	pxe = VIRTUAL( undi.pxs->loader.undi_cs,
+		       undi.pxs->loader.pxe_ptr.offset );
 	printf ( "UNDI driver created a pixie at %hx:%hx...",
-		 undi.pxs->loader.undi_cs, undi.pxs->loader.pxe_off );
+		 undi.pxs->loader.undi_cs, undi.pxs->loader.pxe_ptr.offset );
 	if ( memcmp ( pxe->Signature, "!PXE", 4 ) != 0 ) {
 		printf ( "invalid signature\n" );
 		return 0;
@@ -479,11 +737,13 @@ int undi_loader ( void ) {
 /* Start the UNDI driver.
  */
 
-int eb_pxenv_start_undi ( void ) {
+static int eb_pxenv_start_undi( struct pci_device *pci ) 
+{
 	int success = 0;
 
 	/* AX contains PCI bus:devfn (PCI specification) */
-	undi.pxs->start_undi.ax = ( undi.pci.bus << 8 ) | undi.pci.devfn;
+	undi.pxs->start_undi.ax = ( pci->bus << 8 ) | pci->devfn;
+
 	/* BX and DX set to 0xffff for non-ISAPnP devices
 	 * (BIOS boot specification)
 	 */
@@ -492,8 +752,15 @@ int eb_pxenv_start_undi ( void ) {
 	/* ES:DI points to PnP BIOS' $PnP structure
 	 * (BIOS boot specification)
 	 */
-	undi.pxs->start_undi.es = 0xf000;
-	undi.pxs->start_undi.di = virt_to_phys ( undi.pnp_bios ) - 0xf0000;
+	if ( undi.pnp_bios ) {
+		undi.pxs->start_undi.es = 0xf000;
+		undi.pxs->start_undi.di =
+			virt_to_phys ( undi.pnp_bios ) - 0xf0000;
+	} else {
+		/* Set to a NULL pointer and hope that we don't need it */
+		undi.pxs->start_undi.es = 0x0000;
+		undi.pxs->start_undi.di = 0x0000;
+	}
 
 	DBG ( "PXENV_START_UNDI => AX=%hx BX=%hx DX=%hx ES:DI=%hx:%hx\n",
 	      undi.pxs->start_undi.ax,
@@ -505,7 +772,7 @@ int eb_pxenv_start_undi ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_startup ( void )	{
+static int eb_pxenv_undi_startup ( void )	{
 	int success = 0;
 
 	DBG ( "PXENV_UNDI_STARTUP => (void)\n" );
@@ -515,7 +782,7 @@ int eb_pxenv_undi_startup ( void )	{
 	return success;
 }
 
-int eb_pxenv_undi_cleanup ( void ) {
+static int eb_pxenv_undi_cleanup ( void ) {
 	int success = 0;
 
 	DBG ( "PXENV_UNDI_CLEANUP => (void)\n" );
@@ -524,7 +791,7 @@ int eb_pxenv_undi_cleanup ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_initialize ( void ) {
+static int eb_pxenv_undi_initialize ( void ) {
 	int success = 0;
 
 	undi.pxs->undi_initialize.ProtocolIni = 0;
@@ -537,7 +804,7 @@ int eb_pxenv_undi_initialize ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_shutdown ( void ) {
+static int eb_pxenv_undi_shutdown ( void ) {
 	int success = 0;
 
 	DBG ( "PXENV_UNDI_SHUTDOWN => (void)\n" );
@@ -550,7 +817,7 @@ int eb_pxenv_undi_shutdown ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_open ( void ) {
+static int eb_pxenv_undi_open ( void ) {
 	int success = 0;
 
 	undi.pxs->undi_open.OpenFlag = 0;
@@ -568,7 +835,7 @@ int eb_pxenv_undi_open ( void ) {
 	return success;	
 }
 
-int eb_pxenv_undi_close ( void ) {
+static int eb_pxenv_undi_close ( void ) {
 	int success = 0;
 
 	DBG ( "PXENV_UNDI_CLOSE => (void)\n" );
@@ -578,7 +845,7 @@ int eb_pxenv_undi_close ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_transmit_packet ( void ) {
+static int eb_pxenv_undi_transmit_packet ( void ) {
 	int success = 0;
 	static const uint8_t broadcast[] = { 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF };
 
@@ -626,7 +893,7 @@ int eb_pxenv_undi_transmit_packet ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_set_station_address ( void ) {
+static int eb_pxenv_undi_set_station_address ( void ) {
 	/* This will spuriously fail on some cards.  Ignore failures.
 	 * We only ever use it to set the MAC address to the card's
 	 * permanent value anyway, so it's a useless call (although we
@@ -641,7 +908,7 @@ int eb_pxenv_undi_set_station_address ( void ) {
 	return 1;
 }
 
-int eb_pxenv_undi_get_information ( void ) {
+static int eb_pxenv_undi_get_information ( void ) {
 	int success = 0;
 	memset ( undi.pxs, 0, sizeof ( undi.pxs ) );
 	DBG ( "PXENV_UNDI_GET_INFORMATION => (void)\n" );
@@ -665,7 +932,7 @@ int eb_pxenv_undi_get_information ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_get_iface_info ( void ) {
+static int eb_pxenv_undi_get_iface_info ( void ) {
 	int success = 0;
 
 	DBG ( "PXENV_UNDI_GET_IFACE_INFO => (void)\n" );
@@ -679,7 +946,7 @@ int eb_pxenv_undi_get_iface_info ( void ) {
 	return success;
 }
 
-int eb_pxenv_undi_isr ( void ) {
+static int eb_pxenv_undi_isr ( void ) {
 	int success = 0;
 
 	DBG ( "PXENV_UNDI_ISR => FuncFlag=%hx\n",
@@ -699,7 +966,7 @@ int eb_pxenv_undi_isr ( void ) {
 	return success;
 }
 
-int eb_pxenv_stop_undi ( void ) {
+static int eb_pxenv_stop_undi ( void ) {
 	int success = 0;
 
 	DBG ( "PXENV_STOP_UNDI => (void)\n" );
@@ -709,7 +976,7 @@ int eb_pxenv_stop_undi ( void ) {
 	return success;
 }
 
-int eb_pxenv_unload_stack ( void ) {
+static int eb_pxenv_unload_stack ( void ) {
 	int success = 0;
 
 	memset ( undi.pxs, 0, sizeof ( undi.pxs ) );
@@ -727,7 +994,7 @@ int eb_pxenv_unload_stack ( void ) {
 	return success;
 }
 
-int eb_pxenv_stop_base ( void ) {
+static int eb_pxenv_stop_base ( void ) {
 	int success = 0;
 
 	DBG ( "PXENV_STOP_BASE => (void)\n" );
@@ -738,7 +1005,7 @@ int eb_pxenv_stop_base ( void ) {
 
 /* Unload UNDI base code (if any present) and free memory.
  */
-int undi_unload_base_code ( void ) {
+static int undi_unload_base_code ( void ) {
 	void *bc_code = VIRTUAL( undi.pxe->BC_Code.Seg_Addr, 0 );
 	size_t bc_code_size = undi.pxe->BC_Code.Seg_Size;
 	void *bc_data = VIRTUAL( undi.pxe->BC_Data.Seg_Addr, 0 );
@@ -747,20 +1014,28 @@ int undi_unload_base_code ( void ) {
 	size_t bc_stck_size = undi.pxe->Stack.Seg_Size;
 	firing_squad_lineup_t lineup;
 
-	/* Don't unload if there is no base code present */
-	if ( undi.pxe->BC_Code.Seg_Addr == 0 ) return 1;
-
 	/* Since we never start the base code, the only time we should
 	 * reach this is if we were loaded via PXE.  There are many
 	 * different and conflicting versions of the "correct" way to
 	 * unload the PXE base code, several of which appear within
 	 * the PXE specification itself.  This one seems to work for
 	 * our purposes.
+	 *
+	 * We always call PXENV_STOP_BASE and PXENV_UNLOAD_STACK even
+	 * if the !PXE structure indicates that no base code is
+	 * present.  We do this for the case that there is a
+	 * base-code-less UNDI driver loaded that has hooked some
+	 * interrupts.  If the base code really is absent, then these
+	 * calls will fail, we will ignore the failure, and our
+	 * subsequent memory-freeing code is robust enough to handle
+	 * whatever's thrown at it.
 	 */
 	eb_pxenv_stop_base();
 	eb_pxenv_unload_stack();
 	if ( ( undi.pxs->unload_stack.Status != PXENV_STATUS_SUCCESS ) &&
-	     ( undi.pxs->unload_stack.Status != PXENV_STATUS_FAILURE ) ) {
+	     ( undi.pxs->unload_stack.Status != PXENV_STATUS_FAILURE ) &&
+	     ( undi.pxe->BC_Code.Seg_Addr != 0 ) )
+	{
 		printf ( "Could not free memory allocated to PXE base code: "
 			 "possible memory leak\n" );
 		return 0;
@@ -813,13 +1088,14 @@ int undi_unload_base_code ( void ) {
  * This calls all the various UNDI initialization routines in sequence.
  */
 
-int undi_full_startup ( void ) {
-	if ( ! eb_pxenv_start_undi() ) return 0;
+static int undi_full_startup ( void ) {
 	if ( ! eb_pxenv_undi_startup() ) return 0;
 	if ( ! eb_pxenv_undi_initialize() ) return 0;
 	if ( ! eb_pxenv_undi_get_information() ) return 0;
 	undi.irq = undi.pxs->undi_get_information.IntNumber;
-	if ( ! install_trivial_irq_handler ( undi.irq ) ) {
+	copy_undi_irq_handler ( undi.base_mem_data->irq_handler,
+				UNDI_IRQ_HANDLER_SIZE );
+	if ( ! install_undi_irq_handler ( undi.irq ) ) {
 		undi.irq = IRQ_NONE;
 		return 0;
 	}
@@ -837,7 +1113,7 @@ int undi_full_startup ( void ) {
  * also frees any memory that it can.
  */
 
-int undi_full_shutdown ( void ) {
+static int undi_full_shutdown ( void ) {
 	if ( undi.pxe != NULL ) {
 		/* In case we didn't allocate the driver's memory in the first
 		 * place, try to grab the code and data segments and sizes
@@ -853,7 +1129,7 @@ int undi_full_shutdown ( void ) {
 						   0 );
 			undi.driver_data_size = undi.pxe->UNDIData.Seg_Size;
 		}
-		
+
 		/* Ignore errors and continue in the hope of shutting
 		 * down anyway
 		 */
@@ -870,7 +1146,7 @@ int undi_full_shutdown ( void ) {
 			eb_pxenv_undi_shutdown();
 		}
 		if ( undi.irq != IRQ_NONE ) {
-			remove_trivial_irq_handler ( undi.irq );
+			remove_undi_irq_handler ( undi.irq );
 			undi.irq = IRQ_NONE;
 		}
 		undi_unload_base_code();
@@ -912,7 +1188,7 @@ int undi_full_shutdown ( void ) {
 /**************************************************************************
 POLL - Wait for a frame
 ***************************************************************************/
-static int undi_poll(struct nic *nic)
+static int undi_poll(struct nic *nic, int retrieve)
 {
 	/* Fun, fun, fun.  UNDI drivers don't use polling; they use
 	 * interrupts.  We therefore cheat and pretend that an
@@ -935,8 +1211,22 @@ static int undi_poll(struct nic *nic)
 
 	/* See if a hardware interrupt has occurred since the last poll().
 	 */
-	if ( ! trivial_irq_triggered ( undi.irq ) ) return 0;
+	if ( ! undi_irq_triggered ( undi.irq ) ) return 0;
 
+	/* Given the frailty of PXE stacks, it's probably not safe to
+	 * risk calling PXENV_UNDI_ISR with
+	 * FuncFlag=PXENV_UNDI_ISR_START twice for the same interrupt,
+	 * so we cheat slightly and assume that there is something
+	 * ready to retrieve as long as an interrupt has occurred.
+	 */
+	if ( ! retrieve ) return 1;
+
+#ifdef UNDI_NONTRIVIAL_IRQ
+	/* With the nontrivial IRQ handler, we have already called
+	 * PXENV_UNDI_ISR with PXENV_UNDI_ISR_IN_START and determined
+	 * that it is one of ours.
+	 */
+#else
 	/* Ask the UNDI driver if this is "our" interrupt.
 	 */
 	undi.pxs->undi_isr.FuncFlag = PXENV_UNDI_ISR_IN_START;
@@ -947,11 +1237,7 @@ static int undi_poll(struct nic *nic)
 		 */
 		return 0;
 	}
-
-	/* At this stage, the device should have cleared its interrupt
-	 * line so we can send EOI to the 8259.
-	 */
-	send_specific_eoi ( undi.irq );
+#endif
 
 	/* We might have received a packet, or this might be a
 	 * "transmit completed" interrupt.  Zero nic->packetlen,
@@ -997,15 +1283,12 @@ static int undi_poll(struct nic *nic)
 TRANSMIT - Transmit a frame
 ***************************************************************************/
 static void undi_transmit(
-	struct nic *nic,
+	struct nic *nic __unused,
 	const char *d,			/* Destination */
 	unsigned int t,			/* Type */
 	unsigned int s,			/* size */
 	const char *p)			/* Packet */
 {
-	/* Inhibit compiler warning about unused parameter nic */
-	if ( nic == NULL ) {};
-
 	/* Copy destination to buffer in base memory */
 	memcpy ( undi.xmit_data->destaddr, d, sizeof(MAC_ADDR) );
 
@@ -1040,10 +1323,8 @@ static void undi_transmit(
 /**************************************************************************
 DISABLE - Turn off ethernet interface
 ***************************************************************************/
-static void undi_disable(struct dev *dev)
+static void undi_disable(struct dev *dev __unused)
 {
-	/* Inhibit compiler warning about unused parameter dev */
-	if ( dev == NULL ) {};
 	undi_full_shutdown();
 	free_base_mem_data();
 }
@@ -1057,7 +1338,8 @@ PROBE - Look for an adapter, this routine's visible to the outside
  * to install their drivers.
  */
 
-int hunt_pixies_and_undi_roms ( void ) {
+static int hunt_pixies_and_undi_roms(struct pci_device *pci)
+{
 	static uint8_t hunt_type = HUNT_FOR_PIXIES;
 	
 	if ( hunt_type == HUNT_FOR_PIXIES ) {
@@ -1066,8 +1348,8 @@ int hunt_pixies_and_undi_roms ( void ) {
 		}
 	}
 	hunt_type = HUNT_FOR_UNDI_ROMS;
-	while ( hunt_undi_rom() ) {
-		if ( undi_loader() ) {
+	while (pci && hunt_undi_rom(pci) ) {
+		if ( undi_loader(pci) && eb_pxenv_start_undi(pci) ) {
 			return 1;
 		}
 		undi_full_shutdown(); /* Free any allocated memory */
@@ -1086,22 +1368,25 @@ static int undi_probe(struct dev *dev, struct pci_device *pci)
 	/* Zero out global undi structure */
 	memset ( &undi, 0, sizeof(undi) );
 
-	/* Store PCI parameters; we will need them to initialize the UNDI
-	 * driver later.
-	 */
-	memcpy ( &undi.pci, pci, sizeof(undi.pci) );
-
 	/* Find the BIOS' $PnP structure */
 	if ( ! hunt_pnp_bios() ) {
-		printf ( "No PnP BIOS found; aborting\n" );
-		return 0;
+		/* Not all PXE stacks actually insist on a PnP BIOS.
+		 * In particular, an Etherboot PXE stack will work
+		 * just fine without one.
+		 *
+		 * We used to make this a fatal error, but now we just
+		 * warn and continue.  Note that this is necessary in
+		 * order to be able to debug the Etherboot PXE stack
+		 * under Bochs, since Bochs' BIOS is non-PnP.
+		 */
+		printf ( "WARNING: No PnP BIOS found\n" );
 	}
 
 	/* Allocate base memory data structures */
 	if ( ! allocate_base_mem_data() ) return 0;
 
 	/* Search thoroughly for UNDI drivers */
-	for ( ; hunt_pixies_and_undi_roms(); undi_full_shutdown() ) {
+	for ( ; hunt_pixies_and_undi_roms(pci); undi_full_shutdown() ) {
 		/* Try to initialise UNDI driver */
 		printf ( "Initializing UNDI driver.  Please wait...\n" );
 		if ( ! undi_full_startup() ) {
@@ -1136,6 +1421,12 @@ static int undi_probe(struct dev *dev, struct pci_device *pci)
 	return 0;
 }
 
+static int undi_isa_probe(struct dev *dev, unsigned short *probe_addrs __unused)
+{
+	return undi_probe(dev, NULL);
+}
+
+
 /* UNDI driver states that it is suitable for any PCI NIC (i.e. any
  * PCI device of class PCI_CLASS_NETWORK_ETHERNET).  If there are any
  * obscure UNDI NICs that have the incorrect PCI class, add them to
@@ -1154,4 +1445,11 @@ static struct pci_driver undi_driver __pci_driver = {
 	.class    = PCI_CLASS_NETWORK_ETHERNET,
 };
 
+static struct isa_driver undi_isa_driver __isa_driver = {
+	.type     = NIC_DRIVER,
+	.name     = "UNDI",
+	.probe    = undi_isa_probe,
+	.ioaddrs  = 0,
+};
+
 #endif /* PCBIOS */
