@@ -20,20 +20,73 @@ Literature dealing with the network protocols:
 #include "dev.h"
 #include "nic.h"
 #include "disk.h"
+#include "http.h"
 #include "timer.h"
 #include "cpu.h"
+#include <stdarg.h>
+
+#ifdef CONSOLE_BTEXT
+#include "btext.h"
+#endif
+
+#ifdef CONFIG_FILO
+#include <lib.h>
+#endif
 
 jmp_buf	restart_etherboot;
 int	url_port;		
 
+char as_main_program = 1;
+
 #ifdef	IMAGE_FREEBSD
 int freebsd_howto = 0;
 char freebsd_kernel_env[FREEBSD_KERNEL_ENV_SIZE];
 #endif
 
-#ifdef FREEBSD_PXEEMU
-extern char		pxeemu_nbp_active;
-#endif	/* FREEBSD_PXEBOOT */
+/* in_call(): the entry point to Etherboot.  Generally called from
+ * arch_in_call(), which in turn will have been invoked from
+ * platform-specific assembly code.
+ */
+int in_call ( in_call_data_t *data, uint32_t opcode, va_list params ) {
+	int old_as_main_program = as_main_program;
+	int ret = 0;
+
+	/* Set flat to indicate that we are not running as the main
+	 * program (i.e. we are something like a PXE stack).
+	 */
+	as_main_program = 0;
+
+	/* NOTE: params will cease to be valid if we relocate, since
+	 * it represents a virtual address
+	 */
+	switch ( EB_OPCODE(opcode) ) {
+		
+	case EB_OPCODE_CHECK:
+		/* Installation check
+		 */
+		ret = EB_CHECK_RESULT;
+		break;
+	case EB_OPCODE_MAIN:
+		/* Start up Etherboot as a standalone program. */
+		as_main_program = 1;
+		ret = main ( data, params );
+		break;
+#ifdef PXE_EXPORT
+	case EB_OPCODE_PXE:
+		/* !PXE API call */
+		ret = pxe_in_call ( data, params );
+		break;
+#endif
+	default:
+		printf ( "Unsupported API \"%c%c\"\n",
+			 EB_OPCODE(opcode) >> 8, EB_OPCODE(opcode) & 0xff );
+		ret = -1;
+		break;
+	}
+
+	as_main_program = old_as_main_program;
+	return ret;
+}
 
 static inline unsigned long ask_boot(unsigned *index)
 {
@@ -45,22 +98,22 @@ static inline unsigned long ask_boot(unsigned *index)
 #if defined(ASK_BOOT)
 #if ASK_BOOT >= 0
 	while(1) {
-		int c;
-		unsigned long time;
+		int c = 0;
 		printf(ASK_PROMPT);
 #if ASK_BOOT > 0
-		for (time = currticks() + ASK_BOOT*TICKS_PER_SEC; !iskey(); ) {
-			if (currticks() > time) {
-				c = ANS_DEFAULT;
-				goto done;
+		{
+			unsigned long time;
+			for ( time = currticks() + ASK_BOOT*TICKS_PER_SEC;
+			      !c && !iskey(); ) {
+				if (currticks() > time) c = ANS_DEFAULT;
 			}
 		}
 #endif /* ASK_BOOT > 0 */
-		c = getchar();
+		if ( !c ) c = getchar();
 		if ((c >= 'a') && (c <= 'z')) c &= 0x5F;
 		if ((c >= ' ') && (c <= '~')) putchar(c);
 		putchar('\n');
-done:
+
 		switch(c) {
 		default:
 			/* Nothing useful try again */
@@ -98,7 +151,7 @@ done:
 
 static inline void try_floppy_first(void)
 {
-#if (TRY_FLOPPY_FIRST > 0) && defined(CAN_BOOT_DISK)
+#if (TRY_FLOPPY_FIRST > 0)
 	int i;
 	printf("Trying floppy");
 	disk_init();
@@ -113,7 +166,7 @@ static inline void try_floppy_first(void)
 #endif /* TRY_FLOPPY_FIRST */	
 }
 
-static void console_init(void)
+void console_init(void)
 {
 #ifdef	CONSOLE_SERIAL
 	(void)serial_init();
@@ -121,6 +174,9 @@ static void console_init(void)
 #ifdef 	CONSOLE_DIRECT_VGA
        	video_init();
 #endif
+#ifdef	CONSOLE_BTEXT
+	map_boot_text();
+#endif
 }
 
 static void console_fini(void)
@@ -151,7 +207,7 @@ static int initialized;
 /**************************************************************************
 MAIN - Kick off routine
 **************************************************************************/
-int main(struct Elf_Bhdr *ptr)
+int main(in_call_data_t *data, va_list params)
 {
 	char *p;
 	int state;
@@ -160,11 +216,20 @@ int main(struct Elf_Bhdr *ptr)
 		*p = 0;	/* Zero BSS */
 
 	console_init();
-	arch_main(ptr);
+	arch_main(data,params);
 
-	rom = *(struct rom_info *)phys_to_virt(ROM_INFO_LOCATION);
-	printf("ROM segment %#hx length %#hx reloc %#x\n", rom.rom_segment,
-		rom.rom_length << 1, (unsigned long)_text);
+#if 0
+#ifdef  CONSOLE_BTEXT
+        btext_init();
+        map_boot_text();
+        btext_clearscreen();
+#endif
+#endif 
+
+	if ( rom.rom_segment ) {
+		printf ( "ROM segment %#hx length %#hx reloc %#x\n",
+			 rom.rom_segment, rom.rom_length, _text );
+	}
 
 	cpu_setup();
 	setup_timers();
@@ -243,10 +308,20 @@ static int main_loop(int state)
 			cleanup();
 			console_init();
 			init_heap();
+#ifdef  CONSOLE_BTEXT
+			//I need to all allot
+		        btext_init(); 
+		        map_boot_text();
+		        btext_clearscreen();
+#else
+	#ifdef CONFIG_FILO
+			pci_init();
+	#endif
+#endif
 
 			firsttime = 0;
 		} 
-#ifdef EMERGENCYDISKBOOT
+#ifdef	EXIT_IF_NO_OFFER
 		else {
 			cleanup();
 			exit(0);
@@ -307,7 +382,7 @@ static int main_loop(int state)
 		if (dev->how_probe == PROBE_FAILED) {
 			dev = 0;
 			state = 4;
-		} else if (boot_index && (i == 0) && (boot_index != dev->type_index)) {
+		} else if (boot_index && (i == 0) && (boot_index != (unsigned)dev->type_index)) {
 			printf("Wrong index\n");
 			state = 4;
 		}
@@ -370,6 +445,9 @@ static const struct proto protos[] = {
 #ifdef DOWNLOAD_PROTO_TFTP
 	{ "tftp", tftp },
 #endif
+#ifdef DOWNLOAD_PROTO_HTTP
+        { "http", http },
+#endif
 };
 
 int loadkernel(const char *fname)
@@ -380,27 +458,8 @@ int loadkernel(const char *fname)
 	in_addr ip;
 	int len;
 	const char *name;
-
-#if 0 && defined(CAN_BOOT_DISK)
-	if (!memcmp(fname,"/dev/",5) && fname[6] == 'd') {
-		int dev, part = 0;
-		if (fname[5] == 'f') {
-			if ((dev = fname[7] - '0') < 0 || dev > 3)
-				goto nodisk; }
-		else if (fname[5] == 'h' || fname[5] == 's') {
-			if ((dev = 0x80 + fname[7] - 'a') < 0x80 || dev > 0x83)
-				goto nodisk;
-			if (fname[8]) {
-				part = fname[8] - '0';
-				if (fname[9])
-					part = 10*part + fname[9] - '0'; }
-			/* bootdisk cannot cope with more than eight partitions */
-			if (part < 0 || part > 8)
-				goto nodisk; }
-		else
-			goto nodisk;
-		return(bootdisk(dev, part, load_block));
-	}
+#ifdef	DNS_RESOLVER
+	const char *resolvt;
 #endif
 	ip.s_addr = arptable[ARP_SERVER].ipaddr.s_addr;
 	name = fname;
@@ -417,6 +476,16 @@ int loadkernel(const char *fname)
 	if ((proto < last_proto) && (memcmp(fname + len, "://", 3) == 0)) {
 		name += len + 3;
 		if (name[0] != '/') {
+#ifdef DNS_RESOLVER
+			resolvt = dns_resolver ( name );
+			if ( NULL != resolvt ) {
+				//printf ("Resolved host name [%s] to [%s]\n",
+				//	name, resolvt );
+				inet_aton(resolvt, &ip);
+				while ( ( '/' != name[0] ) && ( 0 != name[0]))
+					++name;
+			} else
+#endif	/* DNS_RESOLVER */
 			name += inet_aton(name, &ip);
 			if (name[0] == ':') {
 				name++;
