@@ -4,10 +4,10 @@ Etherboot -  Network Bootstrap Program
 Literature dealing with the network protocols:
 	ARP - RFC826
 	RARP - RFC903
-        IP - RFC791
+	IP - RFC791
 	UDP - RFC768
 	BOOTP - RFC951, RFC2132 (vendor extensions)
-	DHCP - RFC2131, RFC2132 (options)
+	DHCP - RFC2131, RFC2132, RFC3004 (options)
 	TFTP - RFC1350, RFC2347 (options), RFC2348 (blocksize), RFC2349 (tsize)
 	RPC - RFC1831, RFC1832 (XDR), RFC1833 (rpcbind/portmapper)
 	NFS - RFC1094, RFC1813 (v3, useful for clarifications, not implemented)
@@ -23,8 +23,8 @@ struct arptable_t	arptable[MAX_ARP];
 unsigned long last_igmpv1 = 0;
 struct igmptable_t	igmptable[MAX_IGMP];
 #endif
-/* Currently no other module uses rom, but it is available */
-struct rom_info		rom;
+/* Put rom_info in .nocompress section so romprefix.S can write to it */
+struct rom_info	rom __attribute__ ((section (".text16.nocompress"))) = {0,0};
 static unsigned long	netmask;
 /* Used by nfs.c */
 char *hostname = "";
@@ -39,7 +39,7 @@ static const in_addr zeroIP = { 0L };
 struct bootpd_t bootp_data;
 
 #ifdef	NO_DHCP_SUPPORT
-static unsigned char    rfc1533_cookie[5] = { RFC1533_COOKIE, RFC1533_END };
+static unsigned char	rfc1533_cookie[5] = { RFC1533_COOKIE, RFC1533_END };
 #else	/* !NO_DHCP_SUPPORT */
 static int dhcp_reply;
 static in_addr dhcp_server = { 0L };
@@ -64,10 +64,57 @@ static const unsigned char dhcpdiscover[] = {
 	RFC2132_MSG_TYPE,1,DHCPDISCOVER,
 	RFC2132_MAX_SIZE,2,	/* request as much as we can */
 	ETH_MAX_MTU / 256, ETH_MAX_MTU % 256,
+#ifdef PXE_DHCP_STRICT
+	RFC3679_PXE_CLIENT_UUID,RFC3679_PXE_CLIENT_UUID_LENGTH,RFC3679_PXE_CLIENT_UUID_DEFAULT,
+	RFC3679_PXE_CLIENT_ARCH,RFC3679_PXE_CLIENT_ARCH_LENGTH,RFC3679_PXE_CLIENT_ARCH_IAX86PC,
+	RFC3679_PXE_CLIENT_NDI, RFC3679_PXE_CLIENT_NDI_LENGTH, RFC3679_PXE_CLIENT_NDI_21,
+	RFC2132_VENDOR_CLASS_ID,RFC2132_VENDOR_CLASS_ID_PXE_LENGTH,RFC2132_VENDOR_CLASS_ID_PXE,
+#else
 	RFC2132_VENDOR_CLASS_ID,13,'E','t','h','e','r','b','o','o','t',
 	'-',VERSION_MAJOR+'0','.',VERSION_MINOR+'0',
-	RFC2132_PARAM_LIST,4,RFC1533_NETMASK,RFC1533_GATEWAY,
-	RFC1533_HOSTNAME,RFC1533_VENDOR
+#endif /* PXE_DHCP_STRICT */
+#ifdef DHCP_CLIENT_ID
+	/* Client ID Option */
+	RFC2132_CLIENT_ID, ( DHCP_CLIENT_ID_LEN + 1 ),
+	DHCP_CLIENT_ID_TYPE, DHCP_CLIENT_ID,
+#endif /* DHCP_CLIENT_ID */
+#ifdef DHCP_USER_CLASS
+	/* User Class Option */
+	RFC3004_USER_CLASS, DHCP_USER_CLASS_LEN, DHCP_USER_CLASS,
+#endif /* DHCP_USER_CLASS */
+	RFC2132_PARAM_LIST,
+#define DHCPDISCOVER_PARAMS_BASE 4
+#ifdef  PXE_DHCP_STRICT
+#define DHCPDISCOVER_PARAMS_PXE ( 1 + 8 )
+#else
+#define DHCPDISCOVER_PARAMS_PXE 0
+#endif /* PXE_DHCP_STRICT */
+#ifdef  DNS_RESOLVER
+#define DHCPDISCOVER_PARAMS_DNS  1
+#else
+#define DHCPDISCOVER_PARAMS_DNS  0
+#endif /* DNS_RESOLVER */
+	( DHCPDISCOVER_PARAMS_BASE +
+	  DHCPDISCOVER_PARAMS_PXE+
+	  DHCPDISCOVER_PARAMS_DNS ),
+	RFC1533_NETMASK,
+	RFC1533_GATEWAY,
+	RFC1533_HOSTNAME,
+	RFC1533_VENDOR
+#ifdef PXE_DHCP_STRICT
+	,RFC2132_VENDOR_CLASS_ID,
+	RFC1533_VENDOR_PXE_OPT128,
+	RFC1533_VENDOR_PXE_OPT129,
+	RFC1533_VENDOR_PXE_OPT130,
+	RFC1533_VENDOR_PXE_OPT131,
+	RFC1533_VENDOR_PXE_OPT132,
+	RFC1533_VENDOR_PXE_OPT133,
+	RFC1533_VENDOR_PXE_OPT134,
+	RFC1533_VENDOR_PXE_OPT135
+#endif /* PXE_DHCP_STRICT */
+#ifdef	DNS_RESOLVER
+	,RFC1533_DNS
+#endif
 };
 static const unsigned char dhcprequest [] = {
 	RFC2132_MSG_TYPE,1,DHCPREQUEST,
@@ -75,57 +122,85 @@ static const unsigned char dhcprequest [] = {
 	RFC2132_REQ_ADDR,4,0,0,0,0,
 	RFC2132_MAX_SIZE,2,	/* request as much as we can */
 	ETH_MAX_MTU / 256, ETH_MAX_MTU % 256,
+#ifdef PXE_DHCP_STRICT
+	RFC3679_PXE_CLIENT_UUID,RFC3679_PXE_CLIENT_UUID_LENGTH,RFC3679_PXE_CLIENT_UUID_DEFAULT,
+	RFC3679_PXE_CLIENT_ARCH,RFC3679_PXE_CLIENT_ARCH_LENGTH,RFC3679_PXE_CLIENT_ARCH_IAX86PC,
+	RFC3679_PXE_CLIENT_NDI, RFC3679_PXE_CLIENT_NDI_LENGTH, RFC3679_PXE_CLIENT_NDI_21,
+	RFC2132_VENDOR_CLASS_ID,RFC2132_VENDOR_CLASS_ID_PXE_LENGTH,RFC2132_VENDOR_CLASS_ID_PXE,
+#else
 	RFC2132_VENDOR_CLASS_ID,13,'E','t','h','e','r','b','o','o','t',
 	'-',VERSION_MAJOR+'0','.',VERSION_MINOR+'0',
+#endif /* PXE_DHCP_STRICT */
+#ifdef DHCP_CLIENT_ID
+	/* Client ID Option */
+	RFC2132_CLIENT_ID, ( DHCP_CLIENT_ID_LEN + 1 ),
+	DHCP_CLIENT_ID_TYPE, DHCP_CLIENT_ID,
+#endif /* DHCP_CLIENT_ID */
+#ifdef DHCP_USER_CLASS
+	/* User Class Option */
+	RFC3004_USER_CLASS, DHCP_USER_CLASS_LEN, DHCP_USER_CLASS,
+#endif /* DHCP_USER_CLASS */
 	/* request parameters */
 	RFC2132_PARAM_LIST,
-#ifdef	IMAGE_FREEBSD
-	/* 5 standard + 8 vendortags + 8 motd + 16 menu items */
-	5 + 8 + 8 + 16,
+#define DHCPREQUEST_PARAMS_BASE 5  
+#ifdef  PXE_DHCP_STRICT
+#define DHCPREQUEST_PARAMS_PXE 1
+#define DHCPREQUEST_PARAMS_VENDOR_PXE 8
+#define DHCPREQUEST_PARAMS_VENDOR_EB 0
 #else
-	/* 5 standard + 6 vendortags + 8 motd + 16 menu items */
-	5 + 6 + 8 + 16,
-#endif
-	/* Standard parameters */
-	RFC1533_NETMASK, RFC1533_GATEWAY,
-	RFC1533_HOSTNAME,RFC1533_VENDOR,
+#define DHCPREQUEST_PARAMS_PXE 0
+#define DHCPREQUEST_PARAMS_VENDOR_PXE 0
+#define DHCPREQUEST_PARAMS_VENDOR_EB 4
+#endif /* PXE_DHCP_STRICT */
+#ifdef	IMAGE_FREEBSD
+#define DHCPREQUEST_PARAMS_FREEBSD 2
+#else
+#define DHCPREQUEST_PARAMS_FREEBSD 0
+#endif /* IMAGE_FREEBSD */
+#ifdef  DNS_RESOLVER
+#define DHCPREQUEST_PARAMS_DNS     1
+#else
+#define DHCPREQUEST_PARAMS_DNS     0
+#endif /* DNS_RESOLVER */
+	( DHCPREQUEST_PARAMS_BASE +
+	  DHCPREQUEST_PARAMS_PXE +
+	  DHCPREQUEST_PARAMS_VENDOR_PXE +
+	  DHCPREQUEST_PARAMS_VENDOR_EB +
+	  DHCPREQUEST_PARAMS_DNS +
+	  DHCPREQUEST_PARAMS_FREEBSD ),
+	/* 5 Standard parameters */
+	RFC1533_NETMASK,
+	RFC1533_GATEWAY,
+	RFC1533_HOSTNAME,
+	RFC1533_VENDOR,
 	RFC1533_ROOTPATH,	/* only passed to the booted image */
-	/* Etherboot vendortags */
+#ifndef PXE_DHCP_STRICT
+	/* 4 Etherboot vendortags */
 	RFC1533_VENDOR_MAGIC,
 	RFC1533_VENDOR_ADDPARM,
 	RFC1533_VENDOR_ETHDEV,
 	RFC1533_VENDOR_ETHERBOOT_ENCAP,
+#endif /* ! PXE_DHCP_STRICT */
 #ifdef	IMAGE_FREEBSD
+	/* 2 FreeBSD options */
 	RFC1533_VENDOR_HOWTO,
 	RFC1533_VENDOR_KERNEL_ENV,
 #endif
-	RFC1533_VENDOR_MNUOPTS, RFC1533_VENDOR_SELECTION,
-	/* 8 MOTD entries */
-	RFC1533_VENDOR_MOTD,
-	RFC1533_VENDOR_MOTD+1,
-	RFC1533_VENDOR_MOTD+2,
-	RFC1533_VENDOR_MOTD+3,
-	RFC1533_VENDOR_MOTD+4,
-	RFC1533_VENDOR_MOTD+5,
-	RFC1533_VENDOR_MOTD+6,
-	RFC1533_VENDOR_MOTD+7,
-	/* 16 image entries */
-	RFC1533_VENDOR_IMG,
-	RFC1533_VENDOR_IMG+1,
-	RFC1533_VENDOR_IMG+2,
-	RFC1533_VENDOR_IMG+3,
-	RFC1533_VENDOR_IMG+4,
-	RFC1533_VENDOR_IMG+5,
-	RFC1533_VENDOR_IMG+6,
-	RFC1533_VENDOR_IMG+7,
-	RFC1533_VENDOR_IMG+8,
-	RFC1533_VENDOR_IMG+9,
-	RFC1533_VENDOR_IMG+10,
-	RFC1533_VENDOR_IMG+11,
-	RFC1533_VENDOR_IMG+12,
-	RFC1533_VENDOR_IMG+13,
-	RFC1533_VENDOR_IMG+14,
-	RFC1533_VENDOR_IMG+15,
+#ifdef	DNS_RESOLVER
+	/* 1 DNS option */
+	RFC1533_DNS,
+#endif
+#ifdef  PXE_DHCP_STRICT
+	RFC2132_VENDOR_CLASS_ID,
+	RFC1533_VENDOR_PXE_OPT128,
+	RFC1533_VENDOR_PXE_OPT129,
+	RFC1533_VENDOR_PXE_OPT130,
+	RFC1533_VENDOR_PXE_OPT131,
+	RFC1533_VENDOR_PXE_OPT132,
+	RFC1533_VENDOR_PXE_OPT133,
+	RFC1533_VENDOR_PXE_OPT134,
+	RFC1533_VENDOR_PXE_OPT135,
+#endif /* PXE_DHCP_STRICT */
 };
 
 #ifdef	REQUIRE_VCI_ETHERBOOT
@@ -163,15 +238,19 @@ struct nic	nic =
 		0,				/* type_index */
 		{},				/* state */
 	},
-	(int (*)(struct nic *))dummy,		/* poll */
+	(int (*)(struct nic *, int))dummy,	/* poll */
 	(void (*)(struct nic *, const char *,
 		unsigned int, unsigned int,
 		const char *))dummy,		/* transmit */
+	(void (*)(struct nic *,
+		 irq_action_t))dummy,		/* irq */
 	0,					/* flags */
 	&rom,					/* rom_info */
 	arptable[ARP_CLIENT].node,		/* node_addr */
 	packet + ETH_DATA_ALIGN,		/* packet */
 	0,					/* packetlen */
+	0,					/* ioaddr */
+	0,					/* irqno */
 	0,					/* priv_data */
 };
 
@@ -180,7 +259,7 @@ static int rarp(void);
 #else
 static int bootp(void);
 #endif
-static unsigned short udpchksum(struct iphdr *ip, struct udphdr *udp);
+static unsigned short tcpudpchksum(struct iphdr *ip);
 
 
 int eth_probe(struct dev *dev)
@@ -188,9 +267,9 @@ int eth_probe(struct dev *dev)
 	return probe(dev);
 }
 
-int eth_poll(void)
+int eth_poll(int retrieve)
 {
-	return ((*nic.poll)(&nic));
+	return ((*nic.poll)(&nic, retrieve));
 }
 
 void eth_transmit(const char *d, unsigned int t, unsigned int s, const void *p)
@@ -210,6 +289,11 @@ void eth_disable(void)
 	disable(&nic.dev);
 }
 
+void eth_irq (irq_action_t action)
+{
+	(*nic.irq)(&nic,action);
+}
+
 /*
  * Find out what our boot parameters are
  */
@@ -254,6 +338,10 @@ int eth_load(struct dev *dev __unused)
 			BOOTP_DATA_ADDR->bootp_reply.bp_giaddr.s_addr);
 	if (arptable[ARP_GATEWAY].ipaddr.s_addr)
 		printf(", Gateway %@", arptable[ARP_GATEWAY].ipaddr.s_addr);
+#ifdef	DNS_RESOLVER
+	if (arptable[ARP_NAMESERVER].ipaddr.s_addr)
+		printf(", Nameserver %@", arptable[ARP_NAMESERVER].ipaddr.s_addr);
+#endif
 	putchar('\n');
 
 #ifdef	MDEBUG
@@ -294,10 +382,11 @@ static inline unsigned long default_netmask(void)
 IP_TRANSMIT - Send an IP datagram
 **************************************************************************/
 static int await_arp(int ival, void *ptr,
-	unsigned short ptype, struct iphdr *ip __unused, struct udphdr *udp __unused)
+	unsigned short ptype, struct iphdr *ip __unused, struct udphdr *udp __unused,
+	struct tcphdr *tcp __unused)
 {
 	struct	arprequest *arpreply;
-	if (ptype !=ETH_P_ARP)
+	if (ptype != ETH_P_ARP)
 		return 0;
 	if (nic.packetlen < ETH_HLEN + sizeof(struct arprequest))
 		return 0;
@@ -408,10 +497,32 @@ void build_udp_hdr(unsigned long destip,
 	udp->dest = htons(destsock);
 	udp->len = htons(len - sizeof(struct iphdr));
 	udp->chksum = 0;
-	if ((udp->chksum = udpchksum(ip, udp)) == 0)
+	if ((udp->chksum = tcpudpchksum(ip)) == 0)
 		udp->chksum = 0xffff;
 }
 
+#ifdef DOWNLOAD_PROTO_HTTP
+void build_tcp_hdr(unsigned long destip, unsigned int srcsock,
+		  unsigned int destsock, long send_seq, long recv_seq,
+		  int window, int flags, int ttl, int len, const void *buf)
+{
+       struct iphdr *ip;
+       struct tcphdr *tcp;
+       ip = (struct iphdr *)buf;
+       build_ip_hdr(destip, ttl, IP_TCP, 0, len, buf);
+       tcp = (struct tcphdr *)(ip + 1);
+       tcp->src = htons(srcsock);
+       tcp->dst = htons(destsock);
+       tcp->seq = htonl(send_seq);
+       tcp->ack = htonl(recv_seq);
+       tcp->ctrl = htons(flags + (5 << 12)); /* No TCP options */
+       tcp->window = htons(window);
+       tcp->chksum = 0;
+       if ((tcp->chksum = tcpudpchksum(ip)) == 0)
+	       tcp->chksum = 0xffff;
+}
+#endif
+
 
 /**************************************************************************
 UDP_TRANSMIT - Send an UDP datagram
@@ -423,12 +534,45 @@ int udp_transmit(unsigned long destip, unsigned int srcsock,
 	return ip_transmit(len, buf);
 }
 
+/**************************************************************************
+TCP_TRANSMIT - Send a TCP packet
+**************************************************************************/
+#ifdef DOWNLOAD_PROTO_HTTP
+int tcp_transmit(unsigned long destip, unsigned int srcsock,
+		unsigned int destsock, long send_seq, long recv_seq,
+		int window, int flags, int len, const void *buf)
+{
+       build_tcp_hdr(destip, srcsock, destsock, send_seq, recv_seq,
+		     window, flags, 60, len, buf);
+       return ip_transmit(len, buf);
+}
+
+int tcp_reset(struct iphdr *ip) {
+       struct tcphdr *tcp = (struct tcphdr *)(ip + 1);
+       char buf[sizeof(struct iphdr) + sizeof(struct tcphdr)];
+
+       if (!(tcp->ctrl & htons(RST))) {
+	      long seq = ntohl(tcp->seq) + ntohs(ip->len) -
+			 sizeof(struct iphdr) -
+			 ((ntohs(tcp->ctrl) >> 10) & 0x3C);
+	      if (tcp->ctrl & htons(SYN|FIN))
+		      seq++;
+	      return tcp_transmit(ntohl(ip->src.s_addr),
+				  ntohs(tcp->dst), ntohs(tcp->src),
+				  tcp->ctrl&htons(ACK) ? ntohl(tcp->ack) : 0,
+				  seq, TCP_MAX_WINDOW, RST, sizeof(buf), buf);
+       }
+       return (1);
+}
+#endif
+
 /**************************************************************************
 QDRAIN - clear the nic's receive queue
 **************************************************************************/
 static int await_qdrain(int ival __unused, void *ptr __unused,
 	unsigned short ptype __unused, 
-	struct iphdr *ip __unused, struct udphdr *udp __unused)
+	struct iphdr *ip __unused, struct udphdr *udp __unused,
+	struct tcphdr *tcp __unused)
 {
 	return 0;
 }
@@ -449,7 +593,8 @@ void rx_qdrain(void)
 TFTP - Download extended BOOTP data, or kernel image
 **************************************************************************/
 static int await_tftp(int ival, void *ptr __unused,
-	unsigned short ptype __unused, struct iphdr *ip, struct udphdr *udp)
+	unsigned short ptype __unused, struct iphdr *ip, struct udphdr *udp,
+	struct tcphdr *tcp __unused)
 {
 	if (!udp) {
 		return 0;
@@ -461,150 +606,144 @@ static int await_tftp(int ival, void *ptr __unused,
 	return 1;
 }
 
-int tftp(const char *name, int (*fnc)(unsigned char *, unsigned int, unsigned int, int))
+int tftp ( const char *name,
+	   int (*fnc)(unsigned char *, unsigned int, unsigned int, int) )
 {
-	int             retry = 0;
-	static unsigned short iport = 2000;
-	unsigned short  oport = 0;
-	unsigned short  len, block = 0, prevblock = 0;
-	int		bcounter = 0;
-	struct tftp_t  *tr;
-	struct tftpreq_t tp;
-	int		rc;
-	int		packetsize = TFTP_DEFAULTSIZE_PACKET;
+	struct tftpreq_info_t request_data =
+		{ name, TFTP_PORT, TFTP_MAX_PACKET };
+	struct tftpreq_info_t *request = &request_data;
+	struct tftpblk_info_t block;
+	int rc;
 
-	rx_qdrain();
-
-	tp.opcode = htons(TFTP_RRQ);
-	/* Warning: the following assumes the layout of bootp_t.
-	   But that's fixed by the IP, UDP and BOOTP specs. */
-	len = sizeof(tp.ip) + sizeof(tp.udp) + sizeof(tp.opcode) +
-		sprintf((char *)tp.u.rrq, "%s%coctet%cblksize%c%d",
-		name, 0, 0, 0, TFTP_MAX_PACKET) + 1;
-	if (!udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr, ++iport,
-		TFTP_PORT, len, &tp))
-		return (0);
-	for (;;)
-	{
-		long timeout;
-#ifdef	CONGESTED
-		timeout = rfc2131_sleep_interval(block?TFTP_REXMT: TIMEOUT, retry);
-#else
-		timeout = rfc2131_sleep_interval(TIMEOUT, retry);
-#endif
-		if (!await_reply(await_tftp, iport, NULL, timeout))
-		{
-			if (!block && retry++ < MAX_TFTP_RETRIES)
-			{	/* maybe initial request was lost */
-				if (!udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr,
-					++iport, TFTP_PORT, len, &tp))
-					return (0);
-				continue;
-			}
-#ifdef	CONGESTED
-			if (block && ((retry += TFTP_REXMT) < TFTP_TIMEOUT))
-			{	/* we resend our last ack */
-#ifdef	MDEBUG
-				printf("<REXMT>\n");
-#endif
-				udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr,
-					iport, oport,
-					TFTP_MIN_PACKET, &tp);
-				continue;
-			}
-#endif
-			break;	/* timeout */
-		}
-		tr = (struct tftp_t *)&nic.packet[ETH_HLEN];
-		if (tr->opcode == ntohs(TFTP_ERROR))
-		{
-			printf("TFTP error %d (%s)\n",
-			       ntohs(tr->u.err.errcode),
-			       tr->u.err.errmsg);
-			break;
-		}
-
-		if (tr->opcode == ntohs(TFTP_OACK)) {
-			const char *p = tr->u.oack.data, *e;
-
-			if (prevblock)		/* shouldn't happen */
-				continue;	/* ignore it */
-			len = ntohs(tr->udp.len) - sizeof(struct udphdr) - 2;
-			if (len > TFTP_MAX_PACKET)
-				goto noak;
-			e = p + len;
-			while (*p != '\0' && p < e) {
-				if (!strcasecmp("blksize", p)) {
-					p += 8;
-					if ((packetsize = strtoul(p, &p, 10)) <
-					    TFTP_DEFAULTSIZE_PACKET)
-						goto noak;
-					while (p < e && *p) p++;
-					if (p < e)
-						p++;
-				}
-				else {
-				noak:
-					tp.opcode = htons(TFTP_ERROR);
-					tp.u.err.errcode = 8;
-/*
- *	Warning: the following assumes the layout of bootp_t.
- *	But that's fixed by the IP, UDP and BOOTP specs.
- */
-					len = sizeof(tp.ip) + sizeof(tp.udp) + sizeof(tp.opcode) + sizeof(tp.u.err.errcode) +
-/*
- *	Normally bad form to omit the format string, but in this case
- *	the string we are copying from is fixed. sprintf is just being
- *	used as a strcpy and strlen.
- */
-						sprintf((char *)tp.u.err.errmsg,
-						"RFC1782 error") + 1;
-					udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr,
-						     iport, ntohs(tr->udp.src),
-						     len, &tp);
-					return (0);
-				}
-			}
-			if (p > e)
-				goto noak;
-			block = tp.u.ack.block = 0; /* this ensures, that */
-						/* the packet does not get */
-						/* processed as data! */
-		}
-		else if (tr->opcode == htons(TFTP_DATA)) {
-			len = ntohs(tr->udp.len) - sizeof(struct udphdr) - 4;
-			if (len > packetsize)	/* shouldn't happen */
-				continue;	/* ignore it */
-			block = ntohs(tp.u.ack.block = tr->u.data.block); }
-		else {/* neither TFTP_OACK nor TFTP_DATA */
-			break;
-		}
-
-		if ((block || bcounter) && (block != (unsigned short)(prevblock+1))) {
-			/* Block order should be continuous */
-			tp.u.ack.block = htons(block = prevblock);
-		}
-		tp.opcode = htons(TFTP_ACK);
-		oport = ntohs(tr->udp.src);
-		udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr, iport,
-			oport, TFTP_MIN_PACKET, &tp);	/* ack */
-		if ((unsigned short)(block-prevblock) != 1) {
-			/* Retransmission or OACK, don't process via callback
-			 * and don't change the value of prevblock.  */
-			continue;
-		}
-		prevblock = block;
-		retry = 0;	/* It's the right place to zero the timer? */
-		if ((rc = fnc(tr->u.data.download,
-			      ++bcounter, len, len < packetsize)) <= 0)
-			return(rc);
-		if (len < packetsize) {	/* End of data --- fnc should not have returned */
-			printf("tftp download complete, but\n");
-			return (1);
+	while ( tftp_block ( request, &block ) ) {
+		request = NULL; /* Send request only once */
+		rc = fnc ( block.data, block.block, block.len, block.eof );
+		if ( rc <= 0 ) return (rc);
+		if ( block.eof ) {
+			/* fnc should not have returned */
+			printf ( "TFTP download complete, but\n" );
+			return (0);
 		}
 	}
 	return (0);
 }
+
+int tftp_block ( struct tftpreq_info_t *request, struct tftpblk_info_t *block )
+{
+	static unsigned short lport = 2000; /* local port */
+	static unsigned short rport = TFTP_PORT; /* remote port */
+	struct tftp_t *rcvd = NULL;
+	static struct tftpreq_t xmit;
+	static unsigned short xmitlen = 0;
+	static unsigned short blockidx = 0; /* Last block received */
+	static unsigned short retry = 0; /* Retry attempts on last block */
+	static int blksize = 0;
+	unsigned short recvlen = 0;
+
+	/* If this is a new request (i.e. if name is set), fill in
+	 * transmit block with RRQ and send it.
+	 */
+	if ( request ) {
+		rx_qdrain(); /* Flush receive queue */
+		xmit.opcode = htons(TFTP_RRQ);
+		xmitlen = (void*)&xmit.u.rrq - (void*)&xmit +
+			sprintf((char*)xmit.u.rrq, "%s%coctet%cblksize%c%d",
+				request->name, 0, 0, 0, request->blksize)
+			+ 1; /* null terminator */
+		blockidx = 0; /* Reset counters */
+		retry = 0;
+		blksize = TFTP_DEFAULTSIZE_PACKET;
+		lport++; /* Use new local port */
+		rport = request->port;
+		if ( !udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr, lport,
+				   rport, xmitlen, &xmit) )
+			return (0);
+	}
+	/* Exit if no transfer in progress */
+	if ( !blksize ) return (0);
+	/* Loop to wait until we get a packet we're interested in */
+	block->data = NULL; /* Used as flag */
+	while ( block->data == NULL ) {
+		long timeout = rfc2131_sleep_interval ( blockidx ? TFTP_REXMT :
+							TIMEOUT, retry );
+		if ( !await_reply(await_tftp, lport, NULL, timeout) ) {
+			/* No packet received */
+			if ( retry++ > MAX_TFTP_RETRIES ) break;
+			/* Retransmit last packet */
+			if ( !blockidx ) lport++; /* New lport if new RRQ */
+			if ( !udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr,
+					   lport, rport, xmitlen, &xmit) )
+				return (0);
+			continue; /* Back to waiting for packet */
+		}
+		/* Packet has been received */
+		rcvd = (struct tftp_t *)&nic.packet[ETH_HLEN];
+		recvlen = ntohs(rcvd->udp.len) - sizeof(struct udphdr)
+			- sizeof(rcvd->opcode);
+		rport = ntohs(rcvd->udp.src);
+		retry = 0; /* Reset retry counter */
+		switch ( htons(rcvd->opcode) ) {
+		case TFTP_ERROR : {
+			printf ( "TFTP error %d (%s)\n",
+				 ntohs(rcvd->u.err.errcode),
+				 rcvd->u.err.errmsg );
+			return (0); /* abort */
+		}
+		case TFTP_OACK : {
+			const char *p = rcvd->u.oack.data;
+			const char *e = p + recvlen - 10; /* "blksize\0\d\0" */
+
+			*((char*)(p+recvlen-1)) = '\0'; /* Force final 0 */
+			if ( blockidx || !request ) break; /* Too late */
+			if ( recvlen <= TFTP_MAX_PACKET ) /* sanity */ {
+				/* Check for blksize option honoured */
+				while ( p < e ) {
+					if ( strcasecmp("blksize",p) == 0 &&
+					     p[7] == '\0' ) {
+						blksize = strtoul(p+8,&p,10);
+						p++; /* skip null */
+					}
+					while ( *(p++) ) {};
+				}
+			}
+			if ( blksize < TFTP_DEFAULTSIZE_PACKET || blksize > request->blksize ) {
+				/* Incorrect blksize - error and abort */
+				xmit.opcode = htons(TFTP_ERROR);
+				xmit.u.err.errcode = 8;
+				xmitlen = (void*)&xmit.u.err.errmsg
+					- (void*)&xmit
+					+ sprintf((char*)xmit.u.err.errmsg,
+						  "RFC1782 error")
+					+ 1;
+				udp_transmit(
+				    arptable[ARP_SERVER].ipaddr.s_addr,
+				    lport, rport, xmitlen, &xmit);
+				return (0);
+			}
+		} break;
+		case TFTP_DATA :
+			if ( ntohs(rcvd->u.data.block) != ( blockidx + 1 ) )
+				break; /* Re-ACK last block sent */
+			if ( recvlen > ( blksize+sizeof(rcvd->u.data.block) ) )
+				break; /* Too large; ignore */
+			block->data = rcvd->u.data.download;
+			block->block = ++blockidx;
+			block->len = recvlen - sizeof(rcvd->u.data.block);
+			block->eof = ( (unsigned short)block->len < blksize );
+			/* If EOF, zero blksize to indicate transfer done */
+			if ( block->eof ) blksize = 0;
+			break;
+	        default: break;	/* Do nothing */
+		}
+		/* Send ACK */
+		xmit.opcode = htons(TFTP_ACK);
+		xmit.u.ack.block = htons(blockidx);
+		xmitlen = TFTP_MIN_PACKET;
+		udp_transmit ( arptable[ARP_SERVER].ipaddr.s_addr,
+			       lport, rport, xmitlen, &xmit );
+	}
+	return ( block->data ? 1 : 0 );
+}
 #endif	/* DOWNLOAD_PROTO_TFTP */
 
 #ifdef	RARP_NOT_BOOTP
@@ -612,7 +751,8 @@ int tftp(const char *name, int (*fnc)(unsigned char *, unsigned int, unsigned in
 RARP - Get my IP address and load information
 **************************************************************************/
 static int await_rarp(int ival, void *ptr,
-	unsigned short ptype, struct iphdr *ip, struct udphdr *udp)
+	unsigned short ptype, struct iphdr *ip, struct udphdr *udp,
+	struct tcphdr *tcp __unused)
 {
 	struct arprequest *arpreply;
 	if (ptype != ETH_P_RARP)
@@ -675,7 +815,7 @@ BOOTP - Get my IP address and load information
 **************************************************************************/
 static int await_bootp(int ival __unused, void *ptr __unused,
 	unsigned short ptype __unused, struct iphdr *ip __unused, 
-	struct udphdr *udp)
+	struct udphdr *udp, struct tcphdr *tcp __unused)
 {
 	struct	bootp_t *bootpreply;
 	if (!udp) {
@@ -703,7 +843,7 @@ static int await_bootp(int ival __unused, void *ptr __unused,
 		return 0;
 #ifndef	DEFAULT_BOOTFILE
 	if (bootpreply->bp_file[0] == '\0') {
-		printf("F?");	/* No filename in offer */
+		printf("No filename in DHCP packet\n");	/* No filename in offer */
 		return 0;
 	}
 #endif
@@ -717,7 +857,7 @@ static int await_bootp(int ival __unused, void *ptr __unused,
 #endif	/* NO_DHCP_SUPPORT */
 	netmask = default_netmask();
 	arptable[ARP_SERVER].ipaddr.s_addr = bootpreply->bp_siaddr.s_addr;
-	memset(arptable[ARP_SERVER].node, 0, ETH_ALEN);  /* Kill arp */
+	memset(arptable[ARP_SERVER].node, 0, ETH_ALEN);	 /* Kill arp */
 	arptable[ARP_GATEWAY].ipaddr.s_addr = bootpreply->bp_giaddr.s_addr;
 	memset(arptable[ARP_GATEWAY].node, 0, ETH_ALEN);  /* Kill arp */
 	/* bootpreply->bp_file will be copied to KERNEL_BUF in the memcpy */
@@ -821,7 +961,7 @@ static int bootp(void)
 }
 #endif	/* RARP_NOT_BOOTP */
 
-static uint16_t udpchksum(struct iphdr *ip, struct udphdr *udp)
+static uint16_t tcpudpchksum(struct iphdr *ip)
 {
 	struct udp_pseudo_hdr pseudo;
 	uint16_t checksum;
@@ -829,15 +969,16 @@ static uint16_t udpchksum(struct iphdr *ip, struct udphdr *udp)
 	/* Compute the pseudo header */
 	pseudo.src.s_addr  = ip->src.s_addr;
 	pseudo.dest.s_addr = ip->dest.s_addr;
-	pseudo.unused      = 0;
-	pseudo.protocol    = IP_UDP;
-	pseudo.len         = udp->len;
+	pseudo.unused	   = 0;
+	pseudo.protocol	   = ip->protocol;
+	pseudo.len	   = htons(ntohs(ip->len) - sizeof(struct iphdr));
 
 	/* Sum the pseudo header */
 	checksum = ipchksum(&pseudo, 12);
 
-	/* Sum the rest of the udp packet */
-	checksum = add_ipchksums(12, checksum, ipchksum(udp, ntohs(udp->len)));
+	/* Sum the rest of the tcp/udp packet */
+	checksum = add_ipchksums(12, checksum, ipchksum(ip + 1,
+				 ntohs(ip->len) - sizeof(struct iphdr)));
 	return checksum;
 }
 
@@ -973,11 +1114,245 @@ void join_group(int slot, unsigned long group)
 }
 #else
 #define send_igmp_reports(now) do {} while(0)
-#define process_igmp(ip, now) do {} while(0)
+#define process_igmp(ip, now)  do {} while(0)
 #endif
 
 #include "proto_eth_slow.c"
 
+/**************************************************************************
+TCP - Simple-minded TCP stack. Can only send data once and then
+      receive the response. The algorithm for computing window
+      sizes and delaying ack's is currently broken, and thus
+      disabled. Performance would probably improve a little, if
+      this gets fixed. FIXME
+**************************************************************************/
+#ifdef DOWNLOAD_PROTO_HTTP
+static int await_tcp(int ival, void *ptr, unsigned short ptype __unused,
+		    struct iphdr *ip, struct udphdr *udp __unused,
+		    struct tcphdr *tcp)
+{
+       if (!tcp) {
+	       return 0;
+       }
+       if (arptable[ARP_CLIENT].ipaddr.s_addr != ip->dest.s_addr)
+	       return 0;
+       if (ntohs(tcp->dst) != ival) {
+	       tcp_reset(ip);
+	       return 0;
+       }
+       *(void **)ptr = tcp;
+       return 1;
+}
+
+int tcp_transaction(unsigned long destip, unsigned int destsock, void *ptr,
+		   int (*send)(int len, void *buf, void *ptr),
+		   int (*recv)(int len, const void *buf, void *ptr)) {
+       static uint16_t srcsock = 0;
+       int	       rc = 1;
+       long	       send_seq = currticks();
+       long	       recv_seq = 0;
+       int	       can_send = 0;
+       int	       sent_all = 0;
+       struct iphdr   *ip;
+       struct tcphdr  *tcp;
+       int	       ctrl = SYN;
+       char	       buf[128]; /* Small outgoing buffer */
+       long	       payload;
+       int	       header_size;
+       int	       window = 3*TCP_MIN_WINDOW;
+       long	       last_ack = 0;
+       long	       last_sent = 0;
+       long	       rtt = 0;
+       long	       srtt = 0;
+       long	       rto = TCP_INITIAL_TIMEOUT;
+       int	       retry = TCP_MAX_TIMEOUT/TCP_INITIAL_TIMEOUT;
+       enum { CLOSED, SYN_RCVD, ESTABLISHED,
+	      FIN_WAIT_1, FIN_WAIT_2 } state = CLOSED;
+
+       if (!srcsock) {
+	       srcsock = currticks();
+       }
+       if (++srcsock < 1024)
+	       srcsock += 1024;
+
+       await_reply(await_qdrain, 0, NULL, 0);
+
+ send_data:
+       if (ctrl & ACK)
+	       last_ack = recv_seq;
+       if (!tcp_transmit(destip, srcsock, destsock, send_seq,
+			 recv_seq, window, ctrl,
+			 sizeof(struct iphdr) + sizeof(struct tcphdr)+
+			 can_send, buf)) {
+	       return (0);
+       }
+       last_sent = currticks();
+
+ recv_data:
+       if (!await_reply(await_tcp, srcsock, &tcp,
+			(state == ESTABLISHED && !can_send)
+			? TCP_MAX_TIMEOUT : rto)) {
+	       if (state == ESTABLISHED) {
+ close:
+		       ctrl = FIN|ACK;
+		       state = FIN_WAIT_1;
+		       rc = 0;
+		       goto send_data;
+	       }
+
+	       if (state == FIN_WAIT_1 || state == FIN_WAIT_2)
+		       return (rc);
+
+	       if (--retry <= 0) {
+		       /* time out */
+		       if (state == SYN_RCVD) {
+			       tcp_transmit(destip, srcsock, destsock,
+					    send_seq, 0, window, RST,
+					    sizeof(struct iphdr) +
+					    sizeof(struct tcphdr), buf);
+		       }
+		       return (0);
+	       }
+	       /* retransmit */
+	       goto send_data;
+       }
+ got_data:
+       retry = TCP_MAX_RETRY;
+
+       if (tcp->ctrl & htons(ACK) ) {
+	       char *data;
+	       int syn_ack, consumed;
+
+	       if (state == FIN_WAIT_1 || state == FIN_WAIT_2) {
+		       state = FIN_WAIT_2;
+		       ctrl = ACK;
+		       goto consume_data;
+	       }
+	       syn_ack = state == CLOSED || state == SYN_RCVD;
+	       consumed = ntohl(tcp->ack) - send_seq - syn_ack;
+	       if (consumed < 0 || consumed > can_send) {
+		       tcp_reset((struct iphdr *)&nic.packet[ETH_HLEN]);
+		       goto recv_data;
+	       }
+
+	       rtt = currticks() - last_sent;
+	       srtt = !srtt ? rtt : (srtt*4 + rtt)/5;
+	       rto = srtt + srtt/2;
+	       if (rto < TCP_MIN_TIMEOUT)
+		       rto = TCP_MIN_TIMEOUT;
+	       else if (rto > TCP_MAX_TIMEOUT)
+		       rto = TCP_MAX_TIMEOUT;
+
+	       can_send -= consumed;
+	       send_seq += consumed + syn_ack;
+	       data = buf + sizeof(struct iphdr) + sizeof(struct tcphdr);
+	       if (can_send) {
+		       memmove(data, data + consumed, can_send);
+	       }
+	       if (!sent_all) {
+		       int more_data;
+		       data += can_send;
+		       more_data = buf + sizeof(buf) - data;
+		       if (more_data > 0) {
+			       more_data = send(more_data, data, ptr);
+			       can_send += more_data;
+		       }
+		       sent_all = !more_data;
+	       }
+	       if (state == SYN_RCVD) {
+		       state = ESTABLISHED;
+		       ctrl = PSH|ACK;
+		       goto consume_data;
+	       }
+	       if (tcp->ctrl & htons(RST))
+		       return (0);
+       } else if (tcp->ctrl & htons(RST)) {
+	       if (state == CLOSED)
+		       goto recv_data;
+	       return (0);
+       }
+
+ consume_data:
+       ip  = (struct iphdr *)&nic.packet[ETH_HLEN];
+       header_size = sizeof(struct iphdr) + ((ntohs(tcp->ctrl)>>10)&0x3C);
+       payload = ntohs(ip->len) - header_size;
+       if (payload > 0 && state == ESTABLISHED) {
+	       int old_bytes = recv_seq - (long)ntohl(tcp->seq);
+	       if (old_bytes >= 0 && payload - old_bytes > 0) {
+		       recv_seq += payload - old_bytes;
+		       if (state != FIN_WAIT_1 && state != FIN_WAIT_2 &&
+			   !recv(payload - old_bytes,
+				 &nic.packet[ETH_HLEN+header_size+old_bytes],
+				 ptr)) {
+			       goto close;
+		       }
+		       if ((state == ESTABLISHED || state == SYN_RCVD) &&
+			   !(tcp->ctrl & htons(FIN))) {
+			       int in_window = window - 2*TCP_MIN_WINDOW >
+						recv_seq - last_ack;
+			       ctrl = can_send ? PSH|ACK : ACK;
+			       if (!can_send && in_window) {
+/* Window scaling is broken right now, just fall back to acknowledging every */
+/* packet immediately and unconditionally. FIXME		       */ /***/
+/*				       if (await_reply(await_tcp, srcsock,
+						       &tcp, rto))
+					       goto got_data;
+				       else */
+					       goto send_data;
+			       }
+			       if (!in_window) {
+				       window += TCP_MIN_WINDOW;
+				       if (window > TCP_MAX_WINDOW)
+					       window = TCP_MAX_WINDOW;
+			       }
+			       goto send_data;
+		       }
+	       } else {
+		       /* saw old data again, must have lost packets */
+		       window /= 2;
+		       if (window < 2*TCP_MIN_WINDOW)
+			       window = 2*TCP_MIN_WINDOW;
+	       }
+       }
+
+       if (tcp->ctrl & htons(FIN)) {
+	       if (state == ESTABLISHED) {
+		       ctrl = FIN|ACK;
+	       } else if (state == FIN_WAIT_1 || state == FIN_WAIT_2) {
+		       ctrl = ACK;
+	       } else {
+		       ctrl = RST;
+	       }
+	       return (tcp_transmit(destip, srcsock, destsock,
+				    send_seq, recv_seq + 1, window, ctrl,
+				    sizeof(struct iphdr) +
+				    sizeof(struct tcphdr), buf) &&
+		       (state == ESTABLISHED ||
+			state == FIN_WAIT_1 || state == FIN_WAIT_2) &&
+		       !can_send);
+       }
+
+       if (state == CLOSED) {
+	       if (tcp->ctrl & htons(SYN)) {
+		       recv_seq = ntohl(tcp->seq) + 1;
+		       if (!(tcp->ctrl & htons(ACK))) {
+			       state = SYN_RCVD;
+			       ctrl = SYN|ACK|PSH;
+			       goto send_data;
+		       } else {
+			       state = ESTABLISHED;
+			       ctrl = PSH|ACK;
+		       }
+	       }
+       }
+
+       if (can_send || payload) {
+	       goto send_data;
+       }
+       goto recv_data;
+}
+#endif
+
 /**************************************************************************
 AWAIT_REPLY - Wait until we get a response for our request
 ************f**************************************************************/
@@ -985,21 +1360,22 @@ int await_reply(reply_t reply, int ival, void *ptr, long timeout)
 {
 	unsigned long time, now;
 	struct	iphdr *ip;
-	unsigned iplen;
+	unsigned iplen = 0;
 	struct	udphdr *udp;
+	struct	tcphdr *tcp;
 	unsigned short ptype;
 	int result;
 
 	time = timeout + currticks();
 	/* The timeout check is done below.  The timeout is only checked if
-	 * there is no packet in the Rx queue.  This assumes that eth_poll()
+	 * there is no packet in the Rx queue.	This assumes that eth_poll()
 	 * needs a negligible amount of time.  
 	 */
 	for (;;) {
 		now = currticks();
 		send_eth_slow_reports(now);
 		send_igmp_reports(now);
-		result = eth_poll();
+		result = eth_poll(1);
 		if (result == 0) {
 			/* We don't have anything */
 		
@@ -1064,18 +1440,36 @@ int await_reply(reply_t reply, int ival, void *ptr, long timeout)
 			if (ntohs(udp->len) > (ntohs(ip->len) - iplen))
 				continue;
 
-			if (udp->chksum && udpchksum(ip, udp)) {
+			if (udp->chksum && tcpudpchksum(ip)) {
 				printf("UDP checksum error\n");
 				continue;
 			}
 		}
-		result = reply(ival, ptr, ptype, ip, udp);
+		tcp = 0;
+#ifdef DOWNLOAD_PROTO_HTTP
+		if (ip && (ip->protocol == IP_TCP) &&
+		    (nic.packetlen >=
+		     ETH_HLEN + sizeof(struct iphdr) + sizeof(struct tcphdr))){
+			tcp = (struct tcphdr *)&nic.packet[ETH_HLEN +
+							 sizeof(struct iphdr)];
+			/* Make certain we have a reasonable packet length */
+			if (((ntohs(tcp->ctrl) >> 10) & 0x3C) >
+			    ntohs(ip->len) - (int)iplen)
+				continue;
+			if (tcpudpchksum(ip)) {
+				printf("TCP checksum error\n");
+				continue;
+			}
+
+		}
+#endif
+		result = reply(ival, ptr, ptype, ip, udp, tcp);
 		if (result > 0) {
 			return result;
 		}
 		
 		/* If it isn't a packet the upper layer wants see if there is a default
-		 * action.  This allows us reply to arp and igmp queryies.
+		 * action.  This allows us reply to arp, igmp, and lacp queries.
 		 */
 		if ((ptype == ETH_P_ARP) &&
 			(nic.packetlen >= ETH_HLEN + sizeof(struct arprequest))) {
@@ -1133,8 +1527,8 @@ DECODE_RFC1533 - Decodes RFC1533 header
 int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int eof)
 {
 	static unsigned char *extdata = NULL, *extend = NULL;
-	unsigned char        *extpath = NULL;
-	unsigned char        *endp;
+	unsigned char	     *extpath = NULL;
+	unsigned char	     *endp;
 	static unsigned char in_encapsulated_options = 0;
 
 	if (eof == -1) {
@@ -1143,7 +1537,7 @@ int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int e
 	}
 	else if (block == 0) {
 #ifdef	REQUIRE_VCI_ETHERBOOT
-	vci_etherboot = 0;
+		vci_etherboot = 0;
 #endif
 		end_of_rfc1533 = NULL;
 #ifdef	IMAGE_FREEBSD
@@ -1247,6 +1641,13 @@ int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int e
 					sizeof(freebsd_kernel_env));
 			}
 		}
+#endif
+#ifdef	DNS_RESOLVER
+		else if (NON_ENCAP_OPT c == RFC1533_DNS) {
+			// TODO: Copy the DNS IP somewhere reasonable
+			if (TAG_LEN(p) >= sizeof(in_addr))
+				memcpy(&arptable[ARP_NAMESERVER].ipaddr, p+2, sizeof(in_addr));
+		}
 #endif
 		else {
 #if 0
