@@ -310,6 +310,7 @@ a3c90x_internal_ReadEeprom(int ioaddr, int address)
     }
 
 
+#if 0
 /*** a3c90x_internal_WriteEepromWord - write a physical word of
  *** data to the onboard serial eeprom (not the BIOS prom, but the
  *** nvram in the card that stores, among other things, the MAC
@@ -343,8 +344,9 @@ a3c90x_internal_WriteEepromWord(int ioaddr, int address, unsigned short value)
 
     return 0;
     }
+#endif
 
-
+#if 0
 /*** a3c90x_internal_WriteEeprom - write data to the serial eeprom,
  *** and re-compute the eeprom checksum.
  ***/
@@ -383,8 +385,7 @@ a3c90x_internal_WriteEeprom(int ioaddr, int address, unsigned short value)
 
     return 0;
     }
-
-
+#endif
 
 /*** a3c90x_reset: exported function that resets the card to its default
  *** state.  This is so the Linux driver can re-set the card up the way
@@ -598,7 +599,7 @@ a3c90x_transmit(struct nic *nic __unused, const char *d, unsigned int t,
  *** in nic->packetlen.  Return 1 if a packet was found.
  ***/
 static int
-a3c90x_poll(struct nic *nic)
+a3c90x_poll(struct nic *nic, int retrieve)
     {
     int i, errcode;
 
@@ -607,6 +608,8 @@ a3c90x_poll(struct nic *nic)
 	return 0;
 	}
 
+    if ( ! retrieve ) return 1;
+
     /** we don't need to acknowledge rxComplete -- the upload engine
      ** does it for us.
      **/
@@ -669,7 +672,17 @@ a3c90x_disable(struct dev *dev __unused)
 	outw(cmdTxDisable, INF_3C90X.IOAddr + regCommandIntStatus_w);
 }
 
-
+static void a3c90x_irq(struct nic *nic __unused, irq_action_t action __unused)
+{
+  switch ( action ) {
+  case DISABLE :
+    break;
+  case ENABLE :
+    break;
+  case FORCE :
+    break;
+  }
+}
 
 /*** a3c90x_probe: exported routine to probe for the 3c905 card and perform
  *** initialization.  If this routine is called, the pci functions did find the
@@ -691,6 +704,9 @@ static int a3c90x_probe(struct dev *dev, struct pci_device *pci)
 
     adjust_pci_device(pci);
 
+    nic->ioaddr = pci->ioaddr & ~3;
+    nic->irqno = 0;
+
     INF_3C90X.IOAddr = pci->ioaddr & ~3;
     INF_3C90X.CurrentWindow = 255;
     switch (a3c90x_internal_ReadEeprom(INF_3C90X.IOAddr, 0x03))
@@ -937,6 +953,7 @@ static int a3c90x_probe(struct dev *dev, struct pci_device *pci)
     dev->disable  = a3c90x_disable;
     nic->poll     = a3c90x_poll;
     nic->transmit = a3c90x_transmit;
+    nic->irq      = a3c90x_irq;
 
     return 1;
 }
@@ -959,6 +976,7 @@ PCI_ROM(0x10b7, 0x9058, "3c905b-9058",   "3Com905B-9058"),	/* Cyclone 10/100/BNC
 PCI_ROM(0x10b7, 0x905a, "3c905b-fx",     "3Com905B-FL"),	/* 100 Base FX / 10 Base FX */
 /* Newer 90xC revision: */
 PCI_ROM(0x10b7, 0x9200, "3c905c-tpo",    "3Com905C-TXM"),	/* 10/100 TPO (3C905C-TXM) */
+PCI_ROM(0x10b7, 0x9202, "3c920b-emb-ati", "3c920B-EMB-WNM (ATI Radeon 9100 IGP)"),	/* 3c920B-EMB-WNM (ATI Radeon 9100 IGP) */
 PCI_ROM(0x10b7, 0x9210, "3c920b-emb-wnm","3Com20B-EMB WNM"),
 PCI_ROM(0x10b7, 0x9800, "3c980",         "3Com980-Cyclone"),	/* Cyclone */
 PCI_ROM(0x10b7, 0x9805, "3c9805",        "3Com9805"),		/* Dual Port Server Cyclone */
