@@ -797,7 +797,7 @@ static int forcedeth_reset(struct nic *nic)
 /**************************************************************************
 POLL - Wait for a frame
 ***************************************************************************/
-static int forcedeth_poll(struct nic *nic)
+static int forcedeth_poll(struct nic *nic, int retrieve)
 {
 	/* return true if there's an ethernet packet ready to read */
 	/* nic->packet should contain data on return */
@@ -810,20 +810,23 @@ static int forcedeth_poll(struct nic *nic)
 	i = np->cur_rx % RX_RING;
 	prd = &rx_ring[i];
 
-	if (prd->Flags & cpu_to_le16(NV_RX_DESCRIPTORVALID)) {
-		/* got a valid packet - forward it to the network core */
-		len = cpu_to_le16(prd->Length);
-		nic->packetlen = len;
-		//hex_dump(rxb + (i * RX_NIC_BUFSIZE), len);
-		memcpy(nic->packet, rxb +
-		       (i * RX_NIC_BUFSIZE), nic->packetlen);
-
-		wmb();
-		np->cur_rx++;
-		alloc_rx(nic);
-		return 1;
+	if ( ! (prd->Flags & cpu_to_le16(NV_RX_DESCRIPTORVALID)) ) {
+	  return 0;
 	}
-	return 0;		/* initially as this is called to flush the input */
+
+	if ( ! retrieve ) return 1;
+
+	/* got a valid packet - forward it to the network core */
+	len = cpu_to_le16(prd->Length);
+	nic->packetlen = len;
+	//hex_dump(rxb + (i * RX_NIC_BUFSIZE), len);
+	memcpy(nic->packet, rxb +
+	       (i * RX_NIC_BUFSIZE), nic->packetlen);
+
+	wmb();
+	np->cur_rx++;
+	alloc_rx(nic);
+	return 1;
 }
 
 
@@ -901,20 +904,19 @@ static void forcedeth_disable(struct dev *dev __unused)
 	writel(np->orig_mac[1], base + NvRegMacAddrB);
 }
 
-/**
- * is_valid_ether_addr - Determine if the given Ethernet address is valid
- * @addr: Pointer to a six-byte array containing the Ethernet address
- *
- * Check that the Ethernet address (MAC) is not 00:00:00:00:00:00, is not
- * a multicast address, and is not FF:FF:FF:FF:FF:FF.  The multicast
- * and FF:FF:... tests are combined into the single test "!(addr[0]&1)".
- *
- * Return true if the address is valid.
- */
-static inline int is_valid_ether_addr( u8 *addr ) {
-        const char zaddr[6] = {0,};
-                                                                                
-        return !(addr[0]&1) && memcmp( addr, zaddr, 6);
+/**************************************************************************
+IRQ - Enable, Disable, or Force interrupts
+***************************************************************************/
+static void forcedeth_irq(struct nic *nic __unused, irq_action_t action __unused)
+{
+  switch ( action ) {
+  case DISABLE :
+    break;
+  case ENABLE :
+    break;
+  case FORCE :
+    break;
+  }
 }
 
 /**************************************************************************
@@ -936,6 +938,9 @@ static int forcedeth_probe(struct dev *dev, struct pci_device *pci)
 	printf("forcedeth.c: Found %s, vendor=0x%hX, device=0x%hX\n",
 	       pci->name, pci->vendor, pci->dev_id);
 
+	nic->irqno  = 0;
+	nic->ioaddr = pci->ioaddr & ~3;
+
 	/* point to private storage */
 	np = &npx;
 
@@ -962,17 +967,26 @@ static int forcedeth_probe(struct dev *dev, struct pci_device *pci)
 	nic->node_addr[3] = (np->orig_mac[0] >> 16) & 0xff;
 	nic->node_addr[4] = (np->orig_mac[0] >> 8) & 0xff;
 	nic->node_addr[5] = (np->orig_mac[0] >> 0) & 0xff;
-
-	if (!is_valid_ether_addr(nic->node_addr)) {
+#ifdef LINUX
+	if (!is_valid_ether_addr(dev->dev_addr)) {
 		/*
 		 * Bad mac address. At least one bios sets the mac address
 		 * to 01:23:45:67:89:ab
 		 */
-		printf("Invalid Mac address detected: %!\n",
-		       nic->node_addr);
-		printf("Please complain to your hardware vendor.\n");
-		return 0;
+		printk(KERN_ERR
+		       "%s: Invalid Mac address detected: %02x:%02x:%02x:%02x:%02x:%02x\n",
+		       pci_name(pci_dev), dev->dev_addr[0],
+		       dev->dev_addr[1], dev->dev_addr[2],
+		       dev->dev_addr[3], dev->dev_addr[4],
+		       dev->dev_addr[5]);
+		printk(KERN_ERR
+		       "Please complain to your hardware vendor. Switching to a random MAC.\n");
+		dev->dev_addr[0] = 0x00;
+		dev->dev_addr[1] = 0x00;
+		dev->dev_addr[2] = 0x6c;
+		get_random_bytes(&dev->dev_addr[3], 3);
 	}
+#endif
 	printf("%s: MAC Address %!, ", pci->name, nic->node_addr);
 
 	np->tx_flags =
@@ -1003,6 +1017,7 @@ static int forcedeth_probe(struct dev *dev, struct pci_device *pci)
 	dev->disable = forcedeth_disable;
 	nic->poll = forcedeth_poll;
 	nic->transmit = forcedeth_transmit;
+	nic->irq    = forcedeth_irq;
 	return 1;
 //      }
 	/* else */
