@@ -22,7 +22,7 @@
 *               Written 1999-2002 by Donald Becker
 *
 *               tulip.c: Tulip and Clone Etherboot Driver
-*               By Marty Connor
+*               By Marty Conner
 *               Copyright (C) 2001 Entity Cyber, Inc.
 *
 *    Linux Driver Version LK1.09a, 10-Jul-2003 (2.4.25)
@@ -297,6 +297,9 @@ static struct sundance_private *sdc;
 
 /* Station Address location within the EEPROM */
 #define EEPROM_SA_OFFSET	0x10
+#define DEFAULT_INTR (IntrRxDMADone | IntrPCIErr | \
+                        IntrDrvRqst | IntrTxDone | StatsMax | \
+                        LinkChange)
 
 static int eeprom_read(long ioaddr, int location);
 static int mdio_read(struct nic *nic, int phy_id, unsigned int location);
@@ -434,21 +437,49 @@ static void sundance_reset(struct nic *nic)
 		 (int) inw(BASE + MACCtrl1), (int) inw(BASE + MACCtrl0)));
 }
 
+/**************************************************************************
+IRQ - Wait for a frame
+***************************************************************************/
+void sundance_irq ( struct nic *nic, irq_action_t action ) {
+        unsigned int intr_status;
+
+	switch ( action ) {
+	case DISABLE :
+	case ENABLE :
+		intr_status = inw(nic->ioaddr + IntrStatus);
+		intr_status = intr_status & ~DEFAULT_INTR;
+		if ( action == ENABLE ) 
+			intr_status = intr_status | DEFAULT_INTR;
+		outw(intr_status, nic->ioaddr + IntrEnable);
+		break;
+        case FORCE :
+		outw(0x0200, BASE + ASICCtrl);
+		break;
+        }
+}
 /**************************************************************************
 POLL - Wait for a frame
 ***************************************************************************/
-static int sundance_poll(struct nic *nic)
+static int sundance_poll(struct nic *nic, int retreive)
 {
 	/* return true if there's an ethernet packet ready to read */
 	/* nic->packet should contain data on return */
 	/* nic->packetlen should contain length of data */
 	int entry = sdc->cur_rx % RX_RING_SIZE;
 	u32 frame_status = le32_to_cpu(rx_ring[entry].status);
+	int intr_status;
 	int pkt_len = 0;
 
 	if (!(frame_status & DescOwn))
 		return 0;
 
+	/* There is a packet ready */
+	if(!retreive)
+		return 1;
+
+	intr_status = inw(nic->ioaddr + IntrStatus);
+	outw(intr_status, nic->ioaddr + IntrStatus);
+
 	pkt_len = frame_status & 0x1fff;
 
 	if (frame_status & 0x001f4000) {
@@ -461,12 +492,15 @@ static int sundance_poll(struct nic *nic)
 			nic->packetlen = pkt_len;
 			memcpy(nic->packet, rxb +
 			       (sdc->cur_rx * PKT_BUF_SZ), nic->packetlen);
+
 		}
 	}
 	rx_ring[entry].length = cpu_to_le32(PKT_BUF_SZ | LastFrag);
 	rx_ring[entry].status = 0;
 	entry++;
 	sdc->cur_rx = entry % RX_RING_SIZE;
+	outw(DEFAULT_INTR & ~(IntrRxDone|IntrRxDMADone), 
+		nic->ioaddr + IntrStatus);
 	return 1;
 }
 
@@ -521,7 +555,7 @@ static void sundance_transmit(struct nic *nic, const char *d,	/* Destination */
 /**************************************************************************
 DISABLE - Turn off ethernet interface
 ***************************************************************************/
-static void sundance_disable(struct dev *dev)
+static void sundance_disable(struct dev *dev __unused)
 {
 	/* put the card in its initial state */
 	/* This function serves 3 purposes.
@@ -703,6 +737,9 @@ static int sundance_probe(struct dev *dev, struct pci_device *pci)
 	dev->disable = sundance_disable;
 	nic->poll = sundance_poll;
 	nic->transmit = sundance_transmit;
+	nic->irqno = pci->irq;
+	nic->irq = sundance_irq;
+	nic->ioaddr = BASE;
 
 	return 1;
 }
@@ -711,7 +748,7 @@ static int sundance_probe(struct dev *dev, struct pci_device *pci)
 /* Read the EEPROM and MII Management Data I/O (MDIO) interfaces. */
 static int eeprom_read(long ioaddr, int location)
 {
-	int boguscnt = 2000;	/* Typical 190 ticks */
+	int boguscnt = 10000;	/* Typical 1900 ticks */
 	outw(0x0200 | (location & 0xff), ioaddr + EECtrl);
 	do {
 		if (!(inw(ioaddr + EECtrl) & 0x8000)) {
